@@ -71,7 +71,7 @@ void FMTIn::format(RefFMTNode _t) {
 	switch ( _t->getType()) {
 	case FORMAT:
 	case STRING:
-	case CSTRING:
+	case CSTYLE_STRING:
 	case TL:
 	case TR:
 	case TERM:
@@ -578,7 +578,7 @@ void FMTIn::format_recursive(RefFMTNode _t) {
 	switch ( _t->getType()) {
 	case FORMAT:
 	case STRING:
-	case CSTRING:
+	case CSTYLE_STRING:
 	case TL:
 	case TR:
 	case TERM:
@@ -647,7 +647,7 @@ void FMTIn::format_reversion(RefFMTNode _t) {
 	switch ( _t->getType()) {
 	case FORMAT:
 	case STRING:
-	case CSTRING:
+	case CSTYLE_STRING:
 	case TL:
 	case TR:
 	case TERM:
@@ -1026,6 +1026,7 @@ const char* FMTIn::tokenNames[] = {
 	"TERM",
 	"NONL",
 	"Q",
+	"CSTRING",
 	"T",
 	"X",
 	"A",
@@ -1060,7 +1061,7 @@ const char* FMTIn::tokenNames[] = {
 	"CSF",
 	"NUMBER",
 	"DOT",
-	"CSTRING",
+	"CSTYLE_STRING",
 	"H",
 	"L",
 	"R",
@@ -1071,10 +1072,10 @@ const char* FMTIn::tokenNames[] = {
 	0
 };
 
-const unsigned long FMTIn::_tokenSet_0_data_[] = { 0UL, 134152130UL, 0UL, 0UL };
+const unsigned long FMTIn::_tokenSet_0_data_[] = { 0UL, 268300226UL, 0UL, 0UL };
 // FORMAT STRING "tl" "tr" TERM NONL Q T X A F E SE G SG I O B Z ZZ C 
 const antlr::BitSet FMTIn::_tokenSet_0(_tokenSet_0_data_,4);
-const unsigned long FMTIn::_tokenSet_1_data_[] = { 0UL, 4160758208UL, 4095UL, 0UL, 0UL, 0UL, 0UL, 0UL };
+const unsigned long FMTIn::_tokenSet_1_data_[] = { 0UL, 4026548672UL, 8191UL, 0UL, 0UL, 0UL, 0UL, 0UL };
 // STRING "tl" "tr" X CMOA CMoA CmoA CHI ChI CDWA CDwA CdwA CAPA CApA CapA 
 // CMOI CDI CYI CMI CSI CSF 
 const antlr::BitSet FMTIn::_tokenSet_1(_tokenSet_1_data_,8);
