@@ -60,6 +60,12 @@ namespace lib {
   using namespace std;
   using namespace antlr;
 
+  // AC + JW : to have isinf & isnan ...
+#ifndef _MSC_VER
+  using std::isinf;
+  using std::isnan;
+#endif
+
   void timestamptovalues(EnvT* e) {
 
     //    try {
@@ -119,10 +125,10 @@ namespace lib {
 
         double offset[time_size];
         DDoubleGDL* offset_gdl = new DDoubleGDL(*(new dimension(time_size)));
-      
+
         for(int i = 0 ; i<time_size ; i++) {
           string timestamp = (*timestamps)[i];
-        
+
           if (hasYear) {
             year[i] = stoi(timestamp.substr(0,4));
             (*year_gdl)[i] = year[i];
@@ -139,25 +145,25 @@ namespace lib {
           }
 
           if (hasHour) {
-            if (timestamp.length() > 10) {  
+            if (timestamp.length() > 10) {
               hour[i] = stoi(timestamp.substr(11,2));
-            } else { 
-              hour[i] = 0; 
+            } else {
+              hour[i] = 0;
             }
             (*hour_gdl)[i] = hour[i];
           }
 
           if (hasMinute) {
-            if (timestamp.length() > 10) {  
+            if (timestamp.length() > 10) {
               minute[i] = stoi(timestamp.substr(14,2));
-            } else { 
-              minute[i] = 0; 
+            } else {
+              minute[i] = 0;
             }
             (*minute_gdl)[i] = minute[i];
           }
 
           if (hasSecond) {
-            if (timestamp.length() > 10) {  
+            if (timestamp.length() > 10) {
               second[i] = stoi(timestamp.substr(17,2));
               if (timestamp.substr(19,1) == ".") {
                 if (timestamp.substr(timestamp.length()-1,1) == "Z") {
@@ -166,14 +172,14 @@ namespace lib {
                   second[i] += stod(timestamp.substr(20, timestamp.length()-26))/pow(10,(timestamp.length()-26));
                 }
               }
-            } else { 
-              second[i] = 0; 
+            } else {
+              second[i] = 0;
             }
             (*second_gdl)[i] = second[i];
           }
 
           if (hasOffset) {
-            if (timestamp.length() > 10) {  
+            if (timestamp.length() > 10) {
               string offset_string;
               if (timestamp.substr(timestamp.length()-1,1) == "Z") {
                 offset_string = "+00:00";
@@ -181,8 +187,8 @@ namespace lib {
                 offset_string = timestamp.substr(timestamp.length()-6,7);
               }
               offset[i] = stod(offset_string.substr(0,3)) + stod(offset_string.substr(4,2))/60;
-            } else { 
-            offset[i] = 0; 
+            } else {
+            offset[i] = 0;
             }
             (*offset_gdl)[i] = offset[i];
           }
@@ -242,27 +248,27 @@ namespace lib {
         }
 
         if (hasHour) {
-          if (timestamp.length() > 10) {  
+          if (timestamp.length() > 10) {
             hour = stoi(timestamp.substr(11,2));
-          } else { 
-            hour = 0; 
+          } else {
+            hour = 0;
           }
           DLongGDL* hour_gdl = new DLongGDL(hour);
           e->SetKW(valueHOUR, hour_gdl);
         }
 
         if (hasMinute) {
-          if (timestamp.length() > 10) {  
+          if (timestamp.length() > 10) {
             minute = stoi(timestamp.substr(14,2));
-          } else { 
-            minute = 0; 
+          } else {
+            minute = 0;
           }
           DLongGDL* minute_gdl = new DLongGDL(minute);
           e->SetKW(valueMINUTE, minute_gdl);
         }
 
         if (hasSecond) {
-          if (timestamp.length() > 10) {  
+          if (timestamp.length() > 10) {
             second = stoi(timestamp.substr(17,2));
             if (timestamp.substr(19,1) == ".") {
               if (timestamp.substr(timestamp.length()-1,1) == "Z") {
@@ -271,15 +277,15 @@ namespace lib {
                 second += stod(timestamp.substr(20, timestamp.length()-26))/pow(10,(timestamp.length()-26));
               }
             }
-          } else { 
-            second = 0; 
+          } else {
+            second = 0;
           }
           DDoubleGDL* second_gdl = new DDoubleGDL(second);
           e->SetKW(valueSECOND, second_gdl);
         }
 
         if (hasOffset) {
-          if (timestamp.length() > 10) {  
+          if (timestamp.length() > 10) {
             string offset_string;
             if (timestamp.substr(timestamp.length()-1,1) == "Z") {
               offset_string = "+00:00";
@@ -287,8 +293,8 @@ namespace lib {
               offset_string = timestamp.substr(timestamp.length()-6,7);
             }
             offset = stod(offset_string.substr(0,3)) + stod(offset_string.substr(4,2))/60;
-          } else { 
-          offset = 0; 
+          } else {
+          offset = 0;
           }
           DDoubleGDL* offset_gdl = new DDoubleGDL(offset);
           e->SetKW(valueOFFSET, offset_gdl);
@@ -308,19 +314,19 @@ namespace lib {
     if(isZero) return new DStringGDL("0000-00-00T00:00:00Z");
 
     static int utcIx=e->KeywordIx("UTC");
-    bool isUTC=e->KeywordSet(utcIx);
+    bool isUTC=e->BooleanKeywordAbsentOrSet(utcIx);
 
     struct timeval tv;
-   
+
     /* Get the current time at high precision; could also use clock_gettime() for
      * even higher precision times if we want it. */
     gettimeofday(&tv, NULL);
-   
+
     // convert to time to 'struct tm' for use with strftime
     time_t time_sec=tv.tv_sec;
     struct tm *tm_local=localtime(&time_sec);
     struct tm *tm_utc=gmtime(&time_sec);
-    
+
     if(isUTC){
       tm_local=tm_utc;
     }
@@ -404,13 +410,13 @@ namespace lib {
         strftime(timestamp, sizeof(timestamp), "%Y-%m-%dT%H:%M:%S.000", tm_local);
         //puttinf in milliseconds
         char milli[50]; //size of 50 to avoid buffer overload (not optimal)
-        sprintf(milli, "%03ld", (long) tv.tv_usec/1000);
-        sprintf(timestamp + 20, "%.3s", milli);
+        snprintf(milli,50, "%03ld", (long) tv.tv_usec/1000);
+        snprintf(timestamp + 20,4, "%.3s", milli);
         ts=timestamp;
       }
 
       float diff = 0;
-      if(noOffset){ 
+      if(noOffset){
         //compute difference in hours of local time to UTC time
         diff = (float) tm_local->tm_hour-tm_utc->tm_hour+(tm_local->tm_min-tm_utc->tm_min)/60.0;
       } else {
@@ -425,11 +431,11 @@ namespace lib {
           ts+="Z";
       } else if(h_diff > 0) {
           char diff[50];  //size of 50 to avoid buffer overload (not optimal)
-          sprintf(diff,"%s%02ld:%02ld", "+", h_diff, m_diff);
+          snprintf(diff,50,"%s%02ld:%02ld", "+", h_diff, m_diff);
           ts += diff;
       } else if(h_diff < 0) {
           char diff[50];  //size of 50 to avoid buffer overload (not optimal)
-          sprintf(diff,"%s%02ld:%02ld", "-", -h_diff, m_diff);
+          snprintf(diff,50,"%s%02ld:%02ld", "-", -h_diff, m_diff);
           ts += diff;
       } else {
           ts+="Z";
@@ -445,12 +451,11 @@ namespace lib {
   {
     struct timeval tval;
     struct timezone tzone;
-
-    /*get the time before doing anything else, 
+    
+    /*get the time before doing anything else,
       this hopefully gives a more meaningful "time"
-      than if the t=time(0) call came after an 
+      than if the t=time(0) call came after an
       arbitary number of conditional statements.*/
-    //    cout << "lib::systime: " << t << endl;
     gettimeofday(&tval,&tzone);
     double tt = tval.tv_sec+tval.tv_usec/1e+6; // time in UTC seconds
 
@@ -459,12 +464,12 @@ namespace lib {
 
     Guard<BaseGDL> v_guard;
     Guard<BaseGDL> v1_guard;
-
+    DLong v=0;
     if (nParam == 1) {
-      //1 parameter, 
+      //1 parameter,
       //      1->current UTC time seconds
       //      default
-      DLong v=0; 
+      //DLong v=0;
       e->AssureLongScalarPar(0,v);
 
       //    DIntGDL* v = static_cast<DIntGDL*>(e->GetParDefined(0)->Convert2(GDL_INT,BaseGDL::COPY));
@@ -475,77 +480,98 @@ namespace lib {
       if ((v <= -1) || (v >= 1)) ret_seconds=true;
 
     }
+
+    static double elapsedIx=e->KeywordIx("ELAPSED");
+    //bool iselapsed=e->KeywordSet("ELAPSED");
+    bool iselapsed=e->KeywordPresentAndDefined(elapsedIx);
     static int julianIx=e->KeywordIx("JULIAN");
     bool isjulian=e->KeywordSet(julianIx);
     static int secondsIx=e->KeywordIx("SECONDS");
+    bool isseconds=e->KeywordSet(secondsIx);
     static int utcIx=e->KeywordIx("UTC");
     bool isutc=e->KeywordSet(utcIx);
+    
+    
     if (nParam == 2) {
-      if (isjulian) e->Throw("Conflicting keywords.");
-
+      if (isjulian) e->Throw("Conflicting keywords.");//I don't think it's necessory.
       //2 parameters
       //if the first param is 0, return the date of the second arg
       //if the first param is 1, return the 'double' of the second arg
-      DLong v1=0; 
+      DLong v1=0;
       e->AssureLongScalarPar(0,v1);
-      DDouble v2=0.0; 
-      e->AssureDoubleScalarPar(1,v2);
-
       //     DIntGDL* v1 = static_cast<DIntGDL*>(e->GetParDefined(0)->Convert2(GDL_INT,BaseGDL::COPY));
+
+      if (iselapsed==0){//if is elapsed, v2 is ignored.
+      DDouble v2=0.0;
+      e->AssureDoubleScalarPar(1,v2);
       //v_guard.Reset( v1); //  e->Guard(v1);
       //DDoubleGDL* v2 = static_cast<DDoubleGDL*>(e->GetParDefined(1)->Convert2(GDL_DOUBLE,BaseGDL::COPY));
-
       if (v1 == 0) { //v1->EqualNoDelete( DIntGDL(0))) { //0, read the second argument as time_t;
-	tval.tv_sec = static_cast<long int>(v2);
-        tval.tv_usec = static_cast<long int>((v2-tval.tv_sec)*1e+6);
+          tval.tv_sec = static_cast<long int>(v2);
+          tval.tv_usec = static_cast<long int>((v2-tval.tv_sec)*1e+6);
         //delete v2; // we delete v2 here as it is not guarded. Avoids a "new" in the following "else"
       } else { //1
         return new DDoubleGDL(v2);
       }
+      }
+      else{
+        if ((v1 <= -1) || (v1 >= 1)) ret_seconds=true;
+      }
+      
+    }
+    
+    //
+    if (iselapsed) {
+      DDouble elapsed;
+      e->AssureDoubleScalarKW(elapsedIx, elapsed);
+      tt = elapsed;
+      // In case the elapsed = NaN or Inf;
+      int inputnan = isnan(elapsed);
+      int inputinf = isinf(elapsed);
+      if (inputnan == 1|| inputinf == 1){
+        if (isjulian==1){
+          return new DDoubleGDL(static_cast<double>(tt));
+          }else{
+            if ((ret_seconds|| isseconds )){
+              return new DDoubleGDL(static_cast<double>(tt));
+              } 
+            else { elapsed=0; tt=elapsed;isutc=true;}
+            }
+        }
+      tval.tv_sec = static_cast<long int>(elapsed);
+      tval.tv_usec = static_cast<long int>((elapsed-tval.tv_sec)*1e+6);
     }
 
     //return the variable in seconds, either JULIAN, JULIAN+UTC,
     //or no other keywords
     struct tm *tstruct;
-    if( ret_seconds || e->KeywordSet(secondsIx) )
-      {
-       if( isjulian )
-         {
-           time_t ttime = tval.tv_sec;
-           if( isutc )
-             tstruct=gmtime(&ttime);
-           else
-             tstruct=localtime(&ttime);
-           return new DDoubleGDL(Gregorian2Julian(tstruct));
-         }
-       else
-         {
-           // does not (necessaryly) work: time might count backwards
-           //double tickTime = static_cast<double>(t) + tt - floor( tt);
-           return new DDoubleGDL(static_cast<double>(tt));
-         }
-      }
-
     //return a string of the time, either UTC or local (default)
-    time_t ttime = tval.tv_sec;
-    if(isutc)
-      tstruct= gmtime(&ttime);
-    else
-      tstruct= localtime(&ttime);
 
+    time_t ttime = tval.tv_sec;
+    
+    if(isutc){
+      tstruct= gmtime(&ttime);
+      }
+      else{
+      tstruct= localtime(&ttime);
+      }
     //Convert the time to JULIAN or NOT
-    if(isjulian)
-        return new DDoubleGDL(Gregorian2Julian(tstruct));
-    else 
-      {
+    if(isjulian){
+        DDouble t_nsec = (tt-tval.tv_sec)*1e+9;
+        return new DDoubleGDL(Gregorian2Julian(tstruct,t_nsec));}
+    else if( ret_seconds || isseconds )
+        return new DDoubleGDL(static_cast<double>(tt));
+    else{
        char st[MAX_DATE_STRING_LENGTH];
 //        char *st=new char[MAX_DATE_STRING_LENGTH];
 //        ArrayGuard<char> stGuard( st);
-       const char *format="%a %h %d %T %Y";//my IDL date format.
+       const char *format="%a %h %e %T %Y";//my IDL date format.
        DStringGDL *S;
+    
 #ifdef _WIN32
   char *at = asctime(tstruct); // https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/asctime-wasctime
   at[strlen(at) - 1] = 0;      // On Windows, asctime appends a newline at the end of string.
+  if (at[8] == '0') at[8] = ' '; // convert zero-padded days to space-padded days on Windows
 	S = new DStringGDL(at);
 #else
        SizeT res=strftime(st,MAX_DATE_STRING_LENGTH,format,tstruct);
@@ -560,13 +586,15 @@ namespace lib {
       }
   }
 
+
   BaseGDL* legendre(EnvT* e)
   {
+    SizeT nParam=e->NParam(2); //, "LEGENDRE");
+
     Guard<BaseGDL> x_guard;
     Guard<BaseGDL> l_guard;
     Guard<BaseGDL> m_guard;
 
-    SizeT nParam=e->NParam(2); //, "LEGENDRE");
     Guard<BaseGDL> guard;
     int count;
     
@@ -578,32 +606,48 @@ namespace lib {
     SizeT nEx, nEl, nEm, nmin;
     //    nEl=0;
     //nEm=0;
-
-    nEx=xvals->N_Elements();
-    if(nEx == 0) e->Throw( "Variable is undefined: "+e->GetParString(0));
-
+    if (xvals->Rank() != 0) {// xvals is an array
+      nEx=xvals->N_Elements();
+      if(nEx == 0) e->Throw( "Variable is undefined: "+e->GetParString(0));
+    }else{
+      nEx = 0;
+    }
+    
     //    lvals=e->GetParDefined(1); //,"LEGENDRE");
     lvals= e->GetNumericParDefined(1);
 
-    nEl=lvals->N_Elements();
-    if(nEl == 0) e->Throw("Variable is undefined: "+e->GetParString(1));
+    if (lvals->Rank() != 0) {// lvals is an array
+      nEl=lvals->N_Elements();
+      if(nEl == 0) e->Throw( "Variable is undefined: "+e->GetParString(1));
+    }else{
+      nEl = 0;
+    }
+
+    if(nParam > 2){
+      mvals= e->GetNumericParDefined(2);
+      //mvals=e->GetParDefined(2); //,"LEGENDRE");
+      if (mvals->Rank() != 0) {// lvals is an array
+        nEm=mvals->N_Elements();
+      if(nEm == 0) e->Throw("Variable is undefined: "+e->GetParString(2));
+    }else{nEm = 0;}
+    } else {
+      mvals=new DIntGDL(0);
+	    nEm=0;
+	    guard.Reset(mvals);
+    }
+    
+    nmin=0;
+    if(nEx>0){
+      nmin=nEx;
+      if(nEl < nmin && nEl > 0)         nmin=nEl;
+      if(nEm < nmin && nEm > 0)         nmin=nEm;
+    }else if(nEl>0){
+      nmin=nEl;
+      if(nEm < nmin && nEm > 0)         nmin=nEm;
+    }else if(nEm>0){
+      nmin=nEm;
+    }
     
-    if(nParam > 2)
-      {
-	mvals= e->GetNumericParDefined(2);
-	//mvals=e->GetParDefined(2); //,"LEGENDRE");
-	nEm=mvals->N_Elements();
-      } else {
-	mvals=new DIntGDL(0);
-	nEm=1;
-	guard.Reset(mvals);
-      }
-
-    if(nEm == 0) e->Throw("Variable is undefined: "+e->GetParString(2));
-
-    nmin=nEx;
-    if(nEl < nmin && nEl > 1) 	nmin=nEl;    
-    if(nEm < nmin && nEm > 1) 	nmin=nEm;
     
     if(xvals->Type() == GDL_COMPLEX ||
        xvals->Type() == GDL_COMPLEXDBL) {
@@ -614,6 +658,7 @@ namespace lib {
       DDoubleGDL* res;
       DDoubleGDL* x_cast;
       DIntGDL* l_cast,*m_cast;
+      DDoubleGDL* tep_lcast, *tep_mcast;
 
       if(xvals->Type() == GDL_DOUBLE) 
 	x_cast=  static_cast<DDoubleGDL*>(xvals);
@@ -624,28 +669,25 @@ namespace lib {
 	}
 
       // lvals check
-      if(lvals->Type() == GDL_COMPLEX ||
-	 lvals->Type() == GDL_COMPLEXDBL) 
-	e->Throw( 
-		 "Complex Legendre not implemented: ");
-      else if(lvals->Type() == GDL_INT)
-	l_cast=static_cast<DIntGDL*>(lvals);
-      else
-	{
+    if(lvals->Type() == GDL_COMPLEX || lvals->Type() == GDL_COMPLEXDBL) 
+	  e->Throw( "Complex Legendre not implemented: ");
+    else if(lvals->Type() == GDL_INT){
+	  l_cast=static_cast<DIntGDL*>(lvals);
+    tep_lcast = static_cast<DDoubleGDL*>(lvals->Convert2(GDL_DOUBLE,BaseGDL::COPY));
+      }else{ 
 	  l_cast=static_cast<DIntGDL*>(lvals->Convert2(GDL_INT,BaseGDL::COPY));
 	  l_guard.Reset(l_cast);//e->Guard( l_cast);
+    tep_lcast = static_cast<DDoubleGDL*>(lvals->Convert2(GDL_DOUBLE,BaseGDL::COPY));
 	}
-
       //mval check
-      if(mvals->Type() == GDL_COMPLEX ||
-	 mvals->Type() == GDL_COMPLEXDBL) 
-	e->Throw( 
-		 "Complex Legendre not implemented: ");
-      else if(mvals->Type() == GDL_INT)
-	m_cast=static_cast<DIntGDL*>(mvals);
-      else
-	{
+    if(mvals->Type() == GDL_COMPLEX ||mvals->Type() == GDL_COMPLEXDBL) 
+	    e->Throw("Complex Legendre not implemented: ");
+    else if(mvals->Type() == GDL_INT){
+      m_cast=static_cast<DIntGDL*>(mvals);
+      tep_mcast = static_cast<DDoubleGDL*>(mvals->Convert2(GDL_DOUBLE,BaseGDL::COPY));
+    }else{
 	  m_cast=static_cast<DIntGDL*>(mvals->Convert2(GDL_INT,BaseGDL::COPY));
+    tep_mcast = static_cast<DDoubleGDL*>(mvals->Convert2(GDL_DOUBLE,BaseGDL::COPY));
 	  //e->Guard( m_cast);
 	  m_guard.Reset(m_cast);
 	}
@@ -657,13 +699,27 @@ namespace lib {
       if(nmin == nEx) res=new DDoubleGDL(xvals->Dim(),BaseGDL::NOZERO);
       else if(nmin == nEl) res=new DDoubleGDL(lvals->Dim(),BaseGDL::NOZERO);
       else if(nmin == nEm) res=new DDoubleGDL(mvals->Dim(),BaseGDL::NOZERO);
-	  
+      if(nmin == 0){nmin = 1;}
+
       for (count=0;count<nmin;count++)
 	{
 	  DDouble xNow = (*x_cast)[nmin > nEx?0:count];
 	  DInt lNow =    (*l_cast)[nmin > nEl?0:count];
 	  DInt mNow =    (*m_cast)[nmin > nEm?0:count];
-
+    
+    //JW Feb. 2022: The warnings are not totally the same with IDL, for here it would be more clear; Begin:
+    if( isinf(xNow))
+	    e->Throw( "Argument X can not be infinity.");
+	  if( isnan((*tep_lcast)[nmin > nEl?0:count]))
+	    e->Throw( "Argument L must be a number.");
+    if( isinf((*tep_lcast)[nmin > nEl?0:count]))
+	    e->Throw( "Argument L can not be infinity.");
+	  if( isnan((*tep_mcast)[nmin > nEm?0:count]))
+	    e->Throw( "Argument M must be a number.");
+    if( isinf((*tep_mcast)[nmin > nEm?0:count]))
+	    e->Throw( "Argument M can not be infinity.");
+    // JW end;
+    
 	  if( xNow < -1.0 || xNow > 1.0)
 	    e->Throw( "Argument X must be in the range [-1.0, 1.0]");
 	  if( lNow < 0)
@@ -692,7 +748,7 @@ namespace lib {
 
 	      (*res)[count] = Pm / mul;
 	    }
-	}
+  }
 	
       //convert things back
       static int doubleIx=e->KeywordIx("DOUBLE");
@@ -719,7 +775,7 @@ namespace lib {
     Guard<BaseGDL> fr_guard;
 
     SizeT nParam = e->NParam(1);
-    BaseGDL* v=e->GetNumericParDefined(0);   
+    BaseGDL* v=e->GetNumericParDefined(0);
 
     size_t nEl = v->N_Elements();
     size_t i;
@@ -737,12 +793,12 @@ namespace lib {
 	  new DComplexDblGDL(v->Dim(), BaseGDL::NOZERO);
 	cdr_guard.Reset(cdr);//e->Guard( cdr);
 
-	if(nEl == 1) 
+	if(nEl == 1)
 	  (*cdr)[0]=
 	   DComplex((gsl_sf_exp((*cd)[0].real())*cos((*cd)[0].imag())),
 		    (gsl_sf_exp((*cd)[0].real())*sin((*cd)[0].imag())));
 	else
-	  for(i=0;i<nEl;++i) 
+	  for(i=0;i<nEl;++i)
 	    (*cdr)[i]=
 	      DComplex((gsl_sf_exp((*cd)[i].real())*cos((*cd)[i].imag())),
 		       (gsl_sf_exp((*cd)[i].real())*sin((*cd)[i].imag())));
@@ -757,42 +813,42 @@ namespace lib {
 	DComplexDblGDL* cdr =
 	  new DComplexDblGDL(v->Dim(), BaseGDL::NOZERO);
 
-	if(nEl == 1) 
+	if(nEl == 1)
 	  (*cdr)[0]=
 	   DComplex((gsl_sf_exp((*cd)[0].real())*cos((*cd)[0].imag())),
 		    (gsl_sf_exp((*cd)[0].real())*sin((*cd)[0].imag())));
 	else
-	  for(i=0;i<nEl;i++) 
+	  for(i=0;i<nEl;i++)
 	    (*cdr)[i]=
 	      DComplex((gsl_sf_exp((*cd)[i].real())*cos((*cd)[i].imag())),
 		       (gsl_sf_exp((*cd)[i].real())*sin((*cd)[i].imag())));
-	
+
 	return cdr;
-	
+
       } else if(v->Type() == GDL_DOUBLE) {
-	
-	DDoubleGDL* d=static_cast<DDoubleGDL*>(v->Convert2(GDL_DOUBLE, 
+
+	DDoubleGDL* d=static_cast<DDoubleGDL*>(v->Convert2(GDL_DOUBLE,
 							   BaseGDL::COPY));
 	d_guard.Reset(d);//e->Guard( d);
-	if(nEl == 1) 
+	if(nEl == 1)
 	  (*dr)[0]=gsl_sf_exp((*d)[0]);
 	else
 	  for (i=0;i<nEl;++i) (*dr)[i]=gsl_sf_exp((*d)[i]);
-	
+
 	return dr;
     } else { // all remainding types converted to Float ...
-      /*if(v->Type() == GDL_FLOAT || 
+      /*if(v->Type() == GDL_FLOAT ||
 		v->Type() == GDL_INT ||
 		v->Type() == GDL_LONG) {
       */
 	DFloatGDL *fr=new DFloatGDL(v->Dim(), BaseGDL::NOZERO);
 	fr_guard.Reset(fr);//e->Guard( fr);
 
-	DDoubleGDL* d=static_cast<DDoubleGDL*>(v->Convert2(GDL_DOUBLE, 
+	DDoubleGDL* d=static_cast<DDoubleGDL*>(v->Convert2(GDL_DOUBLE,
 							   BaseGDL::COPY));
 	d_guard.Reset(d);//e->Guard( d);
 
-	if(nEl == 1) 
+	if(nEl == 1)
 	  (*dr)[0]=gsl_sf_exp((*d)[0]);
 	else
 	  for (i=0;i<nEl;++i) (*dr)[i]=gsl_sf_exp((*d)[i]);
@@ -800,7 +856,7 @@ namespace lib {
 	return static_cast<DFloatGDL*>(dr->Convert2(GDL_FLOAT,BaseGDL::COPY));
       }
 
-  
+
     assert(false);
     return NULL;
   }
