@@ -145,13 +145,16 @@ void DCompiler::EndInteractiveStatement() // add common blocks
   ownCommonList.clear(); // not responsible anymore
 }
 
-void DCompiler::StartPro(const string& n, const int compileOpt, const string& o)
+void DCompiler::StartPro(const string& n, const int compileOpt, const string& o, const RefDNode semiCompiledTree)
 {
   ClearOwnCommon();
   if( n != "$MAIN$" || o != "")
     {
       pro = new DPro(n,o,actualFile);
       pro->SetCompileOpt(compileOpt);
+	  //"restored" procedures have no file name associated. They will be marked as "CompileHidden", to avoid the message "Compiled module" issued when (re)compiling the Lexer code.
+	  if (actualFile.length()==0) pro->AddHiddenToCompileOpt();
+	  pro->SetSCC(semiCompiledTree); //entry number
     }
   else
     {
@@ -167,11 +170,14 @@ void DCompiler::ContinueMainPro()
   pro = static_cast<DSubUD*>( env->GetPro());
 }
 
-void DCompiler::StartFun(const string& n, const int compileOpt = 0, const string& o)
+void DCompiler::StartFun(const string& n, const int compileOpt = 0, const string& o, const RefDNode semiCompiledTree)
 {
   ClearOwnCommon();
   pro = new DFun(n,o,actualFile);
   pro->SetCompileOpt(compileOpt);
+  //"restored" procedures have no file name associated. They will be marked as "CompileHidden", to avoid the message "Compiled module" issued when (re)compiling the Lexer code.
+  if (actualFile.length() == 0) pro->AddHiddenToCompileOpt();
+  pro->SetSCC(semiCompiledTree); //entry number
 }
 
 bool DCompiler::IsActivePro( DSub* p)
@@ -245,8 +251,8 @@ void DCompiler::EndPro() // inserts in proList
       }
     }
 
-  if ( subRoutine == "" || subRoutine == pro->ObjectFileName())
-    Message( "Compiled module: "+pro->ObjectName()+"."); 
+    if(!(pro->isHidden() || pro->isGdlHidden())) {  if ( subRoutine == "" || subRoutine == pro->ObjectFileName())
+    Message( "Compiled module: "+pro->ObjectName()+"."); }
 
   // reset pro // will be deleted otherwise
   if( env != NULL) 
@@ -307,12 +313,13 @@ void DCompiler::EndFun() // inserts in funList
   else
   {
     (*searchList).push_back(static_cast<DFun*>(pro));
-    WarnAboutObsoleteRoutine(pro->ObjectName());
+    // sort list again, however item should be inserted at good position in fact (more tricky but would save sort time). TODO.
+   sort( libFunList.begin(), libFunList.end(), CompLibFunName());
+   WarnAboutObsoleteRoutine(pro->ObjectName());
   }
 
-  if (subRoutine == "" || subRoutine == pro->ObjectFileName())
-    Message( "Compiled module: "+pro->ObjectName()+"."); 
-
+  if(!(pro->isHidden() || pro->isGdlHidden())) {  if (subRoutine == "" || subRoutine == pro->ObjectFileName())
+    Message( "Compiled module: "+pro->ObjectName()+"."); }
   // reset pro // will be deleted otherwise
   if( env != NULL) pro=dynamic_cast<DSubUD*>(env->GetPro()); else pro=NULL;
 }
