@@ -21,27 +21,14 @@
 #ifndef HAVE_X
 #else
 
-#include <algorithm>
-#include <vector>
-#include <cstring>
-
-#include <plplot/drivers.h>
-
-#include "graphicsdevice.hpp"
 #include "gdlxstream.hpp"
-#include "initsysvar.hpp"
-#include "gdlexception.hpp"
-
-//defined in graphicsdevice.hpp
-//#define MAX_WIN 32  //IDL free and widgets start at 32 ...
-//#define MAX_WIN_RESERVE 256
 
 class DeviceX : public GraphicsMultiDevice {
     
 public:
     
-    DeviceX() : GraphicsMultiDevice( -1, XC_crosshair, 3, 0) {
-        name = "X";
+    DeviceX(std::string name_="X") : GraphicsMultiDevice( -1, XC_crosshair, 3, 0) {
+        name = name_;
         DLongGDL origin(dimension(2));
         DLongGDL zoom(dimension(2));
         zoom[0] = 1;
@@ -103,18 +90,16 @@ public:
 
     if( wIx >= winList.size() || wIx < 0) return false;
 
-    if( winList[ wIx] != NULL) winList[ wIx]->SetValid(false);
-
-    TidyWindowsList();
+    if( winList[ wIx] != NULL) winList[ wIx]->SetValid(false); TidyWindowsList();
 
     // set initial window size
     PLFLT xp; PLFLT yp; 
     PLINT xleng; PLINT yleng;
     PLINT xoff; PLINT yoff;
 
-    DLong xMaxSize, yMaxSize;
-    DeviceX::MaxXYSize(&xMaxSize, &yMaxSize);
-
+    DLong xMaxSize=640;
+    DLong yMaxSize=512;
+    MaxXYSize(&xMaxSize, &yMaxSize);
     bool noPosx=(xPos==-1);
     bool noPosy=(yPos==-1);
     xPos=max(1,xPos); //starts at 1 to avoid problems plplot!
@@ -125,15 +110,16 @@ public:
 // dynamic allocation needed!    
     PLINT Quadx[4]={xMaxSize-xleng-1,xMaxSize-xleng-1,1,1};
     PLINT Quady[4]={1,yMaxSize-yleng-1,1,yMaxSize-yleng-1};
-    if (noPosx && noPosy) { //no init given, use 4 quadrants:
-      xoff = Quadx[wIx%4];
-      yoff = Quady[wIx%4];
+  int locOnScreen=(wIx>31)?(wIx+2)%4:wIx % 4; //IDL shifts /FREE windows by 2
+  if (noPosx && noPosy) { //no init given, use 4 quadrants:
+      xoff = Quadx[locOnScreen];
+      yoff = Quady[locOnScreen];
     } else if (noPosx) {
-      xoff = Quadx[wIx%4];
+      xoff = Quadx[locOnScreen];
       yoff = yMaxSize-yPos-yleng;
     } else if (noPosy) {
       xoff = xPos;
-      yoff = Quady[wIx%4];
+      yoff = Quady[locOnScreen];
     } else {
       xoff  = xPos;
       yoff  = yMaxSize-yPos-yleng;
@@ -142,72 +128,24 @@ public:
     yoff+=1;
     xp=(*static_cast<DFloatGDL*>( dStruct->GetTag(dStruct->Desc()->TagIndex("X_PX_CM"))))[0]*2.5;
     yp=(*static_cast<DFloatGDL*>( dStruct->GetTag(dStruct->Desc()->TagIndex("Y_PX_CM"))))[0]*2.5;
-    
-    winList[ wIx] = new GDLXStream( xleng, yleng);
-    
+
+    winList[ wIx] = new GDLXStream(xp , yp, xleng, yleng, xoff, yoff, title);
     oList[ wIx]   = oIx++;    
-    winList[ wIx]->spage( xp , yp, xleng, yleng, xoff, yoff); //must be before 'Init'
 
-    // no pause on win destruction
-    winList[ wIx]->spause( false);
-
-    // extended fonts
-    winList[ wIx]->fontld( 1);
-
-    // we want color
-    winList[ wIx]->scolor( 1);
-
-    //if plplot uses a non-staticColor device [TrueColor, StaticColor, StaticGray] 
-    //(because it does not ask for a static even if it exists), we must initialize here colormaps
-    //otherwise our direct X11 color commands work well and are faster than plplot's.
-    if (!this->isStatic()){
-     PLINT r[ctSize], g[ctSize], b[ctSize];
-     GDLCT* myCT=GraphicsDevice::GetGUIDevice( )->GetCT();
-     myCT->Get( r, g, b);
-     winList[ wIx]->scmap0( r, g, b, ctSize); //set colormap 0 to 256 values
-    }
-
-    // window title
-    static char buf[ 256];
-    strncpy( buf, title.c_str(), 255);
-    buf[ 255] = 0;
-//    winList[ wIx]->setopt( "db", 0); //handled elsewhere
-//    winList[ wIx]->setopt( "debug", "1"); //useful for debugging plplot!
-    winList[ wIx]->setopt( "plwindow", buf);
-// Do not init colors --- we handle colors ourseves, very much faster!
-    winList[ wIx]->setopt( "drvopt","noinitcolors=1");
-
-// Please no threads, no gain especially in remote X11 
-   winList[ wIx]->setopt( "drvopt","usepth=0");
-
-//all the options must be passed BEFORE INIT=plinit.
-    winList[ wIx]->Init();
-    // get actual size, and resize to it (overcomes some window managers problems, solves bug #535)
-    // bug #535 had other causes. removed until further notice.
-    //    bool success = WSize( wIx ,&xleng, &yleng, &xoff, &yoff);
-    //    ResizeWin((UInt)xleng, (UInt) yleng);
-    // need to be called initially. permit to fix things
-    winList[ wIx]->ssub(1,1);
-    winList[ wIx]->adv(0);
-    // load font
-    winList[ wIx]->font( 1);
-    winList[ wIx]->vpor(0,1,0,1);
-    winList[ wIx]->wind(0,1,0,1);
-    winList[ wIx]->DefaultCharSize();
-//    //in case these are not initalized, here is a good place to do it.
-//    if (winList[ wIx]->updatePageInfo()==true)
-//    {
-//      winList[ wIx]->GetPlplotDefaultCharSize(); //initializes everything in fact..
-//    }
-    // sets actWin and updates !D
     SetActWin( wIx);
+  //set current cursor:
+  winList[wIx]->CursorStandard(cursorId);
+  winList[wIx]->SetGraphicsFunction(gcFunction);
+  //backingStore (dummy call at the moment)
+  winList[wIx]->SetBackingStore(-1);
+
     bool success;
     if ( hide )
     {
       success=this->Hide();
     }
     else success=this->UnsetFocus();
-    return true; //winList[ wIx]->Valid(); // Valid() need to called once
+    return true; 
     }
   
     GDLGStream* GetStream(bool open = true) {
@@ -216,7 +154,8 @@ public:
             if (!open) return NULL;
 
             DString title = "GDL 0";
-            DLong xSize, ySize;
+            DLong xSize=640;
+            DLong ySize=512;
             DefaultXYSize(&xSize, &ySize);
             bool success = WOpen(0, title, xSize, ySize, -1, -1, false);
             if (!success)
@@ -356,71 +295,39 @@ public:
     return true;
   }
   
-  bool CursorCrosshair()
+  bool CursorCrosshair(bool standard)
   {
-    return CursorStandard(XC_crosshair);
+   if (standard) return CursorStandard(XC_arrow);
+   else return CursorStandard(XC_crosshair);
   }
-  
-  void DefaultXYSize(DLong *xSize, DLong *ySize) {
-    *xSize = 640;
-    *ySize = 512;
 
-    Display* display = XOpenDisplay(NULL);
-    if (display != NULL)
-      {   
-	*xSize = DisplayWidth(display, DefaultScreen(display)) / 2; 
-	*ySize = DisplayHeight(display, DefaultScreen(display)) / 2;
-	XCloseDisplay(display);
-      }   
-
-    bool noQscreen=true;
-    string gdlQscreen=GetEnvString("GDL_GR_X_QSCREEN");
-    if( gdlQscreen == "1") noQscreen=false;
-    string gdlXsize=GetEnvString("GDL_GR_X_WIDTH");
-    if( gdlXsize != "" && noQscreen ) *xSize=atoi(gdlXsize.c_str()); 
-    string gdlYsize=GetEnvString("GDL_GR_X_HEIGHT");
-    if( gdlYsize != "" && noQscreen) *ySize=atoi(gdlYsize.c_str()); 
+ void DefaultXYSize(DLong *xSize, DLong *ySize) {
+  Display* display = XOpenDisplay(NULL);
+  if (display != NULL) {
+   *xSize = DisplayWidth(display, DefaultScreen(display)) / 2;
+   *ySize = DisplayHeight(display, DefaultScreen(display)) / 2;
+   XCloseDisplay(display);
   }
-  
-  void MaxXYSize(DLong *xSize, DLong *ySize) {
-    *ySize = 640;
-    *ySize = 512;
 
-    Display* display = XOpenDisplay(NULL);
-    if (display != NULL)
-      {
-	*xSize = DisplayWidth(display, DefaultScreen(display));
-	*ySize = DisplayHeight(display, DefaultScreen(display));
-	XCloseDisplay(display);
-      }
-    }
-//Please find how to specialize TidyWindowsList for wx and x11 widgets when this function is called
-//as GraphicsDevice::GetDevice()->TidyWindowsList(); which does not return a specialized version.
-// Util then, do not uncomment the following.
-//void TidyWindowsList() {
-//  int wLSize = winList.size();
-//  for (int i = 0; i < wLSize; i++) if (winList[i] != NULL && !winList[i]->GetValid()) {
-//    
-//    //general purpose winlist cleaning with destruction of "closed" plstreams and (eventually) associated widgets:
-//    //X11 case only. For some bad programming reason, sometimes the parent classe (graphicmultidevice) version is 
-//   // used.
-//    delete winList[i];
-//    winList[i] = NULL;
-//    oList[i] = 0;
-//  }
-//  // set new actWin IF NOT VALID ANY MORE
-//  if (actWin < 0 || actWin >= wLSize || winList[actWin] == NULL || !winList[actWin]->GetValid()) {
-//    std::vector< long>::iterator mEl = std::max_element(oList.begin(), oList.end()); // set to most recently created
-//    if (*mEl == 0) { // no window open
-//      SetActWin(-1); //sets   oIx = 1;
-//    } else SetActWin(GraphicsDevice::GetDevice()->GetNonManagedWidgetActWin(false)); //get first non-managed window. false is needed. 
-//  }
-//}  
+  bool noQscreen = true;
+  string gdlQscreen = GetEnvString("GDL_GR_X_QSCREEN");
+  if (gdlQscreen == "1") noQscreen = false;
+  string gdlXsize = GetEnvString("GDL_GR_X_WIDTH");
+  if (gdlXsize != "" && noQscreen) *xSize = atoi(gdlXsize.c_str());
+  string gdlYsize = GetEnvString("GDL_GR_X_HEIGHT");
+  if (gdlYsize != "" && noQscreen) *ySize = atoi(gdlYsize.c_str());
+  }
+
+ void MaxXYSize(DLong *xSize, DLong *ySize) {
+  Display* display = XOpenDisplay(NULL);
+  if (display != NULL) {
+   *xSize = DisplayWidth(display, DefaultScreen(display));
+   *ySize = DisplayHeight(display, DefaultScreen(display));
+   XCloseDisplay(display);
+  }
+ }  
 };
 
-//#undef MAX_WIN
-//#undef MAX_WIN_RESERVE
-
 #endif
 
 #endif
