@@ -22,27 +22,14 @@
 #include "graphicsdevice.hpp"
 #include "gdlgstream.hpp"
 #include "initsysvar.hpp"
+#ifndef MAX
+#define MAX(a,b) ((a) < (b) ? (b) : (a))
+#endif
 
 using namespace std;
 
-// bool GDLGStream::plstreamInitCalled = false;
-
-// void PLPlotAbortHandler(const char *c)
-// {
-//   cout << "PLPlot abort handler: " << c << endl;
-// }
-// 
-// int PLPlotExitHandler(const char *c)
-// {
-//   cout << "PLPlot exit handler: " << c << endl;
-//   return 0;
-// }
-// 
-// void GDLGStream::SetErrorHandlers()
-// {
-//   plsexit( PLPlotExitHandler);
-//   plsabort( PLPlotAbortHandler);
-// }
+static float psCharFudge=1; //to compensate the wrong size of fonts in PS using Hershey
+static float psSymFudge=1; //to compensate the wrong size of symbols in PS
 
 void GDLGStream::Thick(DFloat thick)
 {
@@ -64,25 +51,22 @@ void GDLGStream::Color( ULong color, DLong decomposed) {
     if (decomposed == 0) {
       if (printer && (color & 0xFF) == 0) { color=(bw)?WHITE:BLACK; //note that if bw other colors will be a gray value 
         GDLGStream::SetColorMap1SingleColor(color);
-        plstream::col1(1); //send specifically color ZERO = black.
-        return;
       } else plstream::col0(color & 0xFF); //just set color index [0..255]. simple and fast.
     } else {
       if (printer && color == 0) color=(bw)?WHITE:BLACK;
       GDLGStream::SetColorMap1SingleColor(color);
-      plstream::col1(1); //send specifically color ZERO = black.
-      return;
     }
 }
 #undef BLACK
 
 void GDLGStream::SetColorMap1SingleColor( ULong color)
 {
-    PLINT red[2],green[2],blue[2];
-    red[0] =red[1] = color & 0xFF;
-    green[0] = green[1] =(color >> 8)  & 0xFF;
-    blue[0]= blue[1]=(color >> 16) & 0xFF;
-    SetColorMap1(red, green, blue, 2); 
+    PLINT red[1],green[1],blue[1];
+    red[0] = color & 0xFF;
+    green[0] = (color >> 8)  & 0xFF;
+    blue[0]=(color >> 16) & 0xFF;
+    SetColorMap1(red, green, blue, 1);
+    plstream::col1(0); 
 }
 
 void GDLGStream::SetColorMap1DefaultColors(PLINT ncolors, DLong decomposed)
@@ -102,9 +86,8 @@ void GDLGStream::SetColorMap1DefaultColors(PLINT ncolors, DLong decomposed)
   }
 }
 
-void GDLGStream::SetColorMap1Table( PLINT tableSize, BaseGDL *passed_colors,  DLong decomposed)
+void GDLGStream::SetColorMap1Table( PLINT tableSize, DLongGDL *colors,  DLong decomposed)
 { //cycle on passed colors to fill tableSize.
-  DLongGDL *colors=static_cast<DLongGDL*>(passed_colors);
   DLong n=colors->N_Elements();
 #ifdef _MSC_VER
   PLINT *r = (PLINT*)alloca(sizeof(PLINT)*tableSize);
@@ -134,14 +117,28 @@ void GDLGStream::SetColorMap1Table( PLINT tableSize, BaseGDL *passed_colors,  DL
   SetColorMap1(r, g, b, tableSize); 
 }
 
-void GDLGStream::SetColorMap1Ramp(DLong decomposed, PLFLT minlight)
-{ //cycle on passed colors to fill table1 with ramp.
+DLong GDLGStream::ForceColorMap1Ramp(PLFLT minlight) {
+  DLong old_decomposed=GraphicsDevice::GetDevice()->GetDecomposed();
+  if (old_decomposed == 0) { //just copy Table0 to Table1 so that scale from 0 to 1 in table 1 goes through the whole table
+    PLINT r[ctSize], g[ctSize], b[ctSize];
+    GraphicsDevice::GetCT()->Get(r, g, b);
+    SetColorMap1(r, g, b, ctSize);
+  } else {
+  //force decomposed=false otherwise too difficult with silly plplot colortables.
+  GraphicsDevice::GetDevice()->Decomposed(false);
+  //fill table1 with grey ramp.
     PLFLT h[2], l[2], s[2], pos[2];
-    h[0] = h[1] = s[0] = s[1] = pos[0] = 0.0;
+    bool rev[2];
+    h[0] = h[1] = 0.0;
     l[0] = minlight;
-    l[1] = pos[1] = 1.0;
+    l[1] = 1;
+    s[0] = s[1]= 0.0;
+    rev[0] = rev[1] = false;
+    pos[0] = 0; pos[1]=1;
     SetColorMap1n(256);
-    SetColorMap1l(FALSE,2,pos,h, l, s, NULL); 
+    SetColorMap1l(false,2,pos,h, l, s, rev); 
+  }
+  return old_decomposed;
 }
 #define WHITEB 255
 void GDLGStream::Background( ULong color, DLong decomposed)
@@ -183,28 +180,181 @@ void GDLGStream::DefaultBackground()
   GraphicsDevice::GetDevice()->SetDeviceBckColor( red, green, blue);
 }
 #undef WHITEB
+
+void GDLGStream::SetPageDPMM(float setPsCharFudge, float setPsSymFudge) {
+  //This is supposed to be called each time DPI is changed, which happens only at creation of a new device.
+  // contrary to the page size, that may change for intercative devices that can be resized.
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "SetPageDPMM()\n");
+  psCharFudge=setPsCharFudge;
+  psSymFudge=setPsSymFudge;
+  PLINT level;
+  plstream::glevel(level);
+  if (level <= 1) return;
+  if (thePage.nbPages == 0) return;
+  plstream::ssub(1, 1);
+  plstream::adv(0);
+  PLFLT xdpi, ydpi;
+  PLINT xleng, yleng, xoff, yoff;
+  plstream::gpage( xdpi, ydpi, xleng, yleng, xoff, yoff); //so-called page parameters, the units vary pixels or mm (screen vs. printers)
+//  std::cerr<<"xdpi "<<xdpi<<" ydpi "<<ydpi<<std::endl;
+//  std::cerr<<"xleng "<<xleng<<" yleng "<<yleng<<std::endl;
+  PLFLT charHeight=pls->chrht;
+//  std::cerr<<"charHeight="<<charHeight<<std::endl;
+  thePage.length = xleng;
+  thePage.height = yleng;
+  //get the mm values using gspa:
+  PLFLT bxsize_mm, bysize_mm, offx_mm, offy_mm;
+  PLFLT xmin, ymin, xmax, ymax;
+  plstream::gspa(xmin, xmax, ymin, ymax); //subpage in mm
+  bxsize_mm = xmax - xmin;
+  bysize_mm = ymax - ymin;
+//  std::cerr<<"reported page size inmm "<<bxsize_mm<<","<<bysize_mm<<std::endl;
+  thePage.xsizemm = bxsize_mm;
+  thePage.ysizemm = bysize_mm;
+  //we need to rescale all plplot values such as X_PX_CM, X_CH_SIZE, X_SIZE combinations give the same results as combining plplot values.
+  //the returned xdpi and chrht is not good. We need to adjust chrht
+  offx_mm = xmin;
+  offy_mm = ymin;
+  // test if plplot is consistent:
+  PLFLT theory = xoff / xdpi * 25.4;
+  if (fabs(offx_mm - theory) > 1e-4) if (GDL_DEBUG_PLSTREAM)  fprintf(stderr, "plpot driver returns inconsistent x DPI values.\n");
+  theory = yoff / ydpi * 25.4;
+  if (fabs(offy_mm - theory) > 1e-4) if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "plpot driver returns inconsistent y DPI values.\n");
+  theory = xleng / xdpi * 25.4;
+  if (fabs(bxsize_mm - theory) > 1e-4) if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "plpot driver returns inconsistent x DPI values (2).\n");
+  theory = yleng / ydpi * 25.4;
+  if (fabs(bysize_mm - theory) > 1e-4) if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "plpot driver returns inconsistent y DPI values (2).\n");
+  //we can derive the dpm in x and y which converts mm to device coords:
+  thePage.xdpmm = xdpi/25.4; //abs(thePage.length / bxsize_mm);
+  thePage.ydpmm = ydpi/25.4; //abs(thePage.height / bysize_mm);
+}
+
+bool GDLGStream::updatePageInfo() {
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "updatePageInfo():\n");
+  if (thePage.nbPages == 0) {
+    if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "            FAILED\n");
+    return false;
+  }
+  long xsize, ysize;
+  GetGeometry(xsize, ysize);
+  if (thePage.length == xsize && thePage.height == ysize) return true;
+  thePage.length = xsize;
+  thePage.height = ysize;
+  (*static_cast<DLongGDL*> (SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_SIZE"), 0)))[0] = xsize;
+  (*static_cast<DLongGDL*> (SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_SIZE"), 0)))[0] = ysize;
+  (*static_cast<DLongGDL*> (SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_VSIZE"), 0)))[0] = xsize;
+  (*static_cast<DLongGDL*> (SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_VSIZE"), 0)))[0] = ysize;
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "             %fx%f device units.\n", thePage.length, thePage.height);
+  return true;
+}
+
+void GDLGStream::CurrentCharSize(PLFLT scale) {
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "CurrentCharsize()\n");
+  if (gdlDefaultCharInitialized == 0) {
+    if (updatePageInfo() == true) {
+      GetPlplotDefaultCharSize();
+    }
+  }
+  theCurrentChar.scale = scale;
+  theCurrentChar.ndsx = scale * theDefaultChar.ndsx;
+  theCurrentChar.ndsy = scale * theDefaultChar.ndsy;
+  theCurrentChar.dsx = scale * theDefaultChar.dsx;
+  theCurrentChar.dsy = scale * theDefaultChar.dsy;
+  theCurrentChar.mmsx = scale * theDefaultChar.mmsx;
+  theCurrentChar.mmsy = scale * theDefaultChar.mmsy;
+  theCurrentChar.wsx = scale * theDefaultChar.wsx;
+  theCurrentChar.wsy = scale * theDefaultChar.wsy;
+  theCurrentChar.mmspacing = scale * theDefaultChar.mmspacing;
+  theCurrentChar.nspacing = scale * theDefaultChar.nspacing;
+  theCurrentChar.dspacing = scale * theDefaultChar.dspacing;
+  theCurrentChar.wspacing = scale * theDefaultChar.wspacing;
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "            sized by %f is %fx%f mm or %fx%f device or %fx%f world\n", scale, theCurrentChar.mmsx, theCurrentChar.mmsy, theCurrentChar.dsx, theCurrentChar.dsy, theCurrentChar.wsx, theCurrentChar.wsy);
+}
+
+void GDLGStream::UpdateCurrentCharWorldSize() {
+  PLFLT x, y, dx, dy;
+  DeviceToWorld(0, 0, x, y);
+  DeviceToWorld(theDefaultChar.dsx, theDefaultChar.dsy, dx, dy);
+  theDefaultChar.wsx = abs(dx - x);
+  theDefaultChar.wsy = abs(dy - y);
+  theCurrentChar.wsx = theCurrentChar.scale * theDefaultChar.wsx;
+  theCurrentChar.wsy = theCurrentChar.scale * theDefaultChar.wsy;
+
+  DeviceToWorld(0, theDefaultChar.dspacing, dx, dy);
+  theDefaultChar.wspacing = abs(dy - y);
+
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "UpdateCurrentCharWorldSize(%f,%f)\n",
+    theCurrentChar.wsx, theCurrentChar.wsy);
+}
+
+void GDLGStream::updateBoxDeviceCoords() {
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "updateBoxDeviceCoords()\n");
+  // world coordinates of current subpage boundaries and page boundaries
+  NormedDeviceToWorld(0.0, 0.0, theBox.pageWorldCoordinates[0], theBox.pageWorldCoordinates[2]);
+  NormedDeviceToWorld(1.0, 1.0, theBox.pageWorldCoordinates[1], theBox.pageWorldCoordinates[3]);
+  NormToWorld(0.0, 0.0, theBox.subPageWorldCoordinates[0], theBox.subPageWorldCoordinates[2]);
+  NormToWorld(1.0, 1.0, theBox.subPageWorldCoordinates[1], theBox.subPageWorldCoordinates[3]);
+  NormToDevice(theBox.nx1, theBox.ny1, theBox.dx1, theBox.dy1);
+  NormToDevice(theBox.nx2, theBox.ny2, theBox.dx2, theBox.dy2);
+  theBox.dxsize=theBox.dx2-theBox.dx1;
+  theBox.dysize=theBox.dy2-theBox.dy1;
+}
+void GDLGStream::syncPageInfo()
+{
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "SyncPageInfo()\n");
+  PLINT level;
+  plstream::glevel(level);
+  if (level > 1 && thePage.nbPages != 0) //we need to have a vpor defined, and a page!
+  {
+    thePage.subpage.dxoff = 0; //our subpages have 0 offset
+    thePage.subpage.dyoff = 0;
+    thePage.subpage.dxsize = thePage.length / thePage.nx;
+    thePage.subpage.dysize = thePage.height / thePage.ny;
+    if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "         subpage is %fx%f at [%f,%f] device units\n",
+      thePage.subpage.dxsize, thePage.subpage.dysize, thePage.subpage.dxoff, thePage.subpage.dyoff);
+
+  } else if (GDL_DEBUG_PLSTREAM) fprintf(stderr, "       WARNING: not initalized\n");
+}
 void GDLGStream::DefaultCharSize() {
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"GDLGStream::DefaultCharSize()\n");
   DStructGDL* d = SysVar::D();
   DStructDesc* s = d->Desc();
   int X_CH_SIZE = s->TagIndex("X_CH_SIZE");
   int Y_CH_SIZE = s->TagIndex("Y_CH_SIZE");
-  int X_PX_CM = s->TagIndex("X_PX_CM");
-  int Y_PX_CM = s->TagIndex("Y_PX_CM");
-  DLong chx = (*static_cast<DLongGDL*> (d->GetTag(X_CH_SIZE, 0)))[0];
-  DLong chy = (*static_cast<DLongGDL*> (d->GetTag(Y_CH_SIZE, 0)))[0];
-  DFloat xpxcm = (*static_cast<DFloatGDL*> (d->GetTag(X_PX_CM, 0)))[0];
-  DFloat ypxcm = (*static_cast<DFloatGDL*> (d->GetTag(Y_PX_CM, 0)))[0];
-  DFloat xchsizemm = GetPlplotFudge() * chx * CM_IN_MM / xpxcm;
-  DFloat linespacingmm = GetPlplotFudge() * chy * CM_IN_MM / ypxcm;
-  schr(xchsizemm, 1.0, linespacingmm);
-}
-  void GDLGStream::RenewPlplotDefaultCharsize(PLFLT newMmSize)
-  {
-    plstream::schr(newMmSize, 1.0);
-    gdlDefaultCharInitialized=0;
-    GetPlplotDefaultCharSize();
+  DLong ichx = (*static_cast<DLongGDL*> (d->GetTag(X_CH_SIZE, 0)))[0];
+  DLong ichy = (*static_cast<DLongGDL*> (d->GetTag(Y_CH_SIZE, 0)))[0];
+  PLFLT chx=ichx; //needed as floats for subsequent computations!
+  PLFLT chy=ichy;
+  int FLAGS = s->TagIndex("FLAGS");
+  DLong flags = (*static_cast<DLongGDL*> (d->GetTag(FLAGS, 0)))[0];
+  if (flags & 0x1) {
+    int X_PX_CM = s->TagIndex("X_PX_CM");
+    int Y_PX_CM = s->TagIndex("Y_PX_CM");
+    DFloat xpxcm = (*static_cast<DFloatGDL*> (d->GetTag(X_PX_CM, 0)))[0];
+    DFloat ypxcm = (*static_cast<DFloatGDL*> (d->GetTag(Y_PX_CM, 0)))[0];
+    setVariableCharacterSize(chx, 1.0, chy,xpxcm,ypxcm);
+  } else {
+    setFixedCharacterSize(chx, 1.0, chy);
   }
-  
+}
+void GDLGStream::SetCharSize(DLong ichx, DLong chy) {
+  PLFLT chx=ichx;
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"GDLGStream::SetCharSize()\n");
+  DStructGDL* d = SysVar::D();
+  DStructDesc* s = d->Desc();
+  int FLAGS = s->TagIndex("FLAGS");
+  DLong flags = (*static_cast<DLongGDL*> (d->GetTag(FLAGS, 0)))[0];
+  if (flags & 0x1) {
+    int X_PX_CM = s->TagIndex("X_PX_CM");
+    int Y_PX_CM = s->TagIndex("Y_PX_CM");
+    DFloat xpxcm = (*static_cast<DFloatGDL*> (d->GetTag(X_PX_CM, 0)))[0];
+    DFloat ypxcm = (*static_cast<DFloatGDL*> (d->GetTag(Y_PX_CM, 0)))[0];
+    setVariableCharacterSize(chx, 1.0, chy,xpxcm,ypxcm);
+  } else {
+    setFixedCharacterSize(chx, 1.0, chy);
+  }
+}
+ 
   void GDLGStream::GetPlplotDefaultCharSize()
   {
         
@@ -215,7 +365,6 @@ void GDLGStream::DefaultCharSize() {
     theDefaultChar.scale=1.0;
     theDefaultChar.mmsx=pls->chrht; //millimeter
     theDefaultChar.mmsy=pls->chrht;
-    theDefaultChar.fudge=GetPlplotFudge();
     theDefaultChar.ndsx=mm2ndx(theDefaultChar.mmsx); //normalized device
     theDefaultChar.ndsy=mm2ndy(theDefaultChar.mmsy);
     theDefaultChar.dsy=theDefaultChar.ndsy*thePage.height;
@@ -234,8 +383,9 @@ void GDLGStream::DefaultCharSize() {
     gdlDefaultCharInitialized=1;
   }
 
-void GDLGStream::NextPlot( bool erase )
-{
+void GDLGStream::NextPlot( bool erase ) {
+  // restore charsize to default for newpage at beginning since adv() uses charsize to get box position.
+  if (!erase) sizeChar(1.0);
   DLongGDL* pMulti = SysVar::GetPMulti();
 
   DLong nx = (*pMulti)[ 1];
@@ -250,8 +400,9 @@ void GDLGStream::NextPlot( bool erase )
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"NextPlot(erase=%d)\n",erase);
   // set subpage numbers in X and Y
 //  plstream::ssub( nx, ny ); // ssub does not change charsize it seems
-  ssub( nx, ny ); 
-  DLong pMod = (*pMulti)[0] % (nx*ny);
+  ssub( nx, ny, nz ); 
+  DLong nsub=nx*ny*nz;
+  DLong pMod = (*pMulti)[0] % (nsub);
 
 //  if( (*pMulti)[0] <= 0 || (*pMulti)[0] == nx*ny) // clear and restart to first subpage
   if( pMod == 0 ) // clear and restart to first subpage
@@ -278,18 +429,18 @@ void GDLGStream::NextPlot( bool erase )
 
 //    plstream::adv(1); //advance to first subpage
     adv(1); //advance to first subpage
-    (*pMulti)[0] = nx*ny*nz-1; //set PMULTI[0] to this page
+    (*pMulti)[0] = nsub-1; //set PMULTI[0] to this page
   }
   else
   {
     if( dir == 0 )
     {
 //      plstream::adv(nx*ny - pMod + 1);
-      adv(nx*ny - pMod + 1);
+      adv(nsub - pMod + 1);
     }
     else
     {
-      int p = nx*ny - pMod;
+      int p = nsub - pMod;
       int pp = p*nx % (nx*ny) + p/ny + 1;
 //      plstream::adv(pp);
       adv(pp);
@@ -299,8 +450,7 @@ void GDLGStream::NextPlot( bool erase )
       --(*pMulti)[0];
     }
   }
-  // restore charsize to default for newpage
-  sizeChar(1.0);
+
 }
 
 void GDLGStream::NoSub()
@@ -325,16 +475,8 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
   
 //since the page sizes for PS and EPS images are processed by GDL after plplot finishes 
 //its work, gpage will not output correct sizes 
-  DString name = (*static_cast<DStringGDL*>(
-    SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("NAME"), 0)
-  ))[0];
-  if (name == "PS") { 
-    xSize = (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_SIZE"), 0)))[0];
-    ySize = (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_SIZE"), 0)))[0];
-  } else {
   xSize = xleng;
   ySize = yleng;
-  }
   if (xSize<1.0||ySize<1) //plplot gives back crazy values! z-buffer for example!
   {
     PLFLT xmin,xmax,ymin,ymax;
@@ -366,7 +508,7 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
   // - ... a look-up table instead of the long switch/case blocks ...
  
   size_t len = strlen(in);
-  if (stringLength) *stringLength=0;
+  if (stringLength!=NULL) *stringLength=0;
   // skip conversion if the string is empty
   if (len == 0) return "";
 
@@ -463,10 +605,6 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
           curr_fnt = 12; 
           break;
 
-        case 'C' : case 'c' : // carriage return (TODO FIXME: does not work in PostScript!)
-          out += "#[0xD]";
-          break;
-
         case 'X' : case 'x' : // reversion to entry font 
           out += internalFontCodes[curr_fnt = next_fnt = default_fnt];
           break;
@@ -559,12 +697,12 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
     }
     else 
     {
-      if (stringLength) *stringLength+=base*fact[curr_lev%7];
+      if (stringLength!=NULL) *stringLength+=base*fact[curr_lev%7];
       curr_pos++;
       // handling IDL exclamation mark escape '!!'
       if (in[i] == '!') {
         i++;
-        if (stringLength) *stringLength+=base*fact[curr_lev%7];
+        if (stringLength!=NULL) *stringLength+=base*fact[curr_lev%7];
       }
       // handling plplot number sign escape '##'
       if 
@@ -894,12 +1032,12 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
   activeFontCodeNum = curr_fnt;
   //if gdlGetStringLength function is available, use it to give back a better value ("X" and "I" do not have the same width in hershey format!)
 #if PLPLOT_PRIVATE_NOT_HIDDEN
-  if (stringLength) *stringLength=gdlGetStringLength(out)/this->mmCharLength();
+  if (stringLength!=NULL) *stringLength=gdlGetStringLength(out)/this->mmCharLength();
 #endif
   return out;
 retrn:
   activeFontCodeNum = curr_fnt;
-  if (stringLength) *stringLength=0;
+  if (stringLength!=NULL) *stringLength=0;
   cout << "ERROR: GDLGStream::TranslateFormatCodes(\"" << in << "\") = \"" << out << "\"" << endl;  
   return ""; 
 }
@@ -907,8 +1045,8 @@ retrn:
 void GDLGStream::setSymbolSize( PLFLT scale )
 {
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"setSymbolScale(%f)\n",scale);
-  plstream::ssym(0.0, scale);
-  theCurrentSymSize=scale;
+//  plstream::ssym(0.0, scale);
+  theCurrentSymSize=scale*psSymFudge;
 }
 
 void GDLGStream::setLineSpacing(PLFLT newSpacing)
@@ -916,42 +1054,184 @@ void GDLGStream::setLineSpacing(PLFLT newSpacing)
   theLineSpacing_in_mm=newSpacing;
 }
 PLFLT GDLGStream::getSymbolSize(){return theCurrentSymSize;}
-void GDLGStream::mtex( const char *side, PLFLT disp, PLFLT pos, PLFLT just,
-                       const char *text)
+void GDLGStream::mtex( const char *side, PLFLT disp, PLFLT posit, PLFLT just,
+                       const char *text, double *stringCharLength, double *stringCharHeight)
 {
-  plstream::mtex(side,disp,pos,just,TranslateFormatCodes(text).c_str());
+   //plot does not handle !C
+  size_t len = strlen(text);
+  bool simple=true;
+  for (size_t i = 0; i < len; i++) if (text[i] == '!' && text[i + 1] == 'C') {
+    simple=false;
+  }
+  if (simple) {
+    plstream::mtex(side,disp,posit,just,TranslateFormatCodes(text,stringCharLength).c_str());
+	if (stringCharHeight!=NULL) *stringCharHeight = 1;
+    return;
+  }
+  //complicated:
+  if (stringCharHeight != NULL) *stringCharHeight = 0;
+  double d=0;
+  std::string s(text);
+  std::string newline="!C";
+  size_t pos = 0, oldpos=0;
+  PLFLT yadd=nLineSpacing()/nCharHeight();
+  std::vector<long> positions;
+  while (pos != string::npos) {
+    pos = s.find(newline, oldpos);
+    while (text[pos-1] == '!') { pos = s.find(newline, pos+2); }
+    positions.push_back(oldpos);
+    positions.push_back(pos);
+    oldpos=pos+2;
+  }
+  for (std::vector<long>::iterator it = positions.begin(); it != positions.end();) {
+    oldpos=(*it++);
+    pos=(*(it++));
+    size_t l=pos-oldpos; 
+    if (l<0) l=string::npos;
+//    std::cerr<<pos<<":"<<l<<" "<<s.substr(oldpos,l)<<std::endl;
+    plstream::mtex(side,disp,posit,just,TranslateFormatCodes(s.substr(oldpos,l).c_str(),&d).c_str());
+    if (strstr(side,"b")!=NULL) { //bottom increments 1 line
+	  disp += yadd;
+	} else if (strstr(side,"t")!=NULL) {//top decrements 1 line
+	  disp -= yadd;
+	} else {//left decrements position in Y : change posit not disp unless parallel
+	  if (strstr(side,"v")!=NULL) posit-=nLineSpacing()/boxnYSize() ; else disp -= yadd;
+	}
+	if (stringCharLength!=NULL) *stringCharLength = std::max<double>(*stringCharLength, d);
+	if (stringCharHeight!=NULL) *stringCharHeight += 1;
+  }
 }
 
-void GDLGStream::mtex3( const char *side, PLFLT disp, PLFLT pos, PLFLT just,
-                       const char *text)
-{
-  plstream::mtex3(side,disp,pos,just,TranslateFormatCodes(text).c_str());
-}
 void GDLGStream::ptex( PLFLT x, PLFLT y, PLFLT dx, PLFLT dy, PLFLT just,
                        const char *text , double *stringCharLength)
 {
-  plstream::ptex(x,y,dx,dy,just,TranslateFormatCodes(text,stringCharLength).c_str());
+  //plot does not handle !C
+  size_t len = strlen(text);
+  bool simple=true;
+  for (size_t i = 0; i < len; i++) if (text[i] == '!' && text[i + 1] == 'C') {
+    simple=false;
+  }
+  if (simple) {
+    plstream::ptex(x,y,dx,dy,just,TranslateFormatCodes(text,stringCharLength).c_str());
+    return;
+  }
+  //complicated:
+  double d=0;
+  std::string s(text);
+  std::string newline="!C";
+  size_t pos = 0, oldpos=0;
+  PLFLT ydisp=(nLineSpacing()/nCharHeight())*wCharHeight();
+  std::vector<long> positions;
+  while (pos != string::npos) {
+    pos = s.find(newline, oldpos);
+    while (text[pos-1] == '!') { pos = s.find(newline, pos+2); }
+    positions.push_back(oldpos);
+    positions.push_back(pos);
+    oldpos=pos+2;
+  }
+  for (std::vector<long>::iterator it = positions.begin(); it != positions.end();) {
+    oldpos=(*it++);
+    pos=(*(it++));
+    size_t l=pos-oldpos; 
+    if (l<0) l=string::npos;
+//    std::cerr<<pos<<":"<<l<<" "<<s.substr(oldpos,l)<<std::endl;
+    plstream::ptex(x,y,dx,dy,just,TranslateFormatCodes(s.substr(oldpos,l).c_str(),&d).c_str()) ;
+    y-=ydisp;
+    *stringCharLength=std::max<double>(*stringCharLength,d);
+  }
 }
 
-void GDLGStream::schr( PLFLT charwidthmm, PLFLT scale , PLFLT lineSpacingmm)
+//This defines the character size for SCALABLE character devices (POSTSCRIPT, SVG))
+//The dimension of "average" character (given by X_CH_SIZE) is to be a physical (mm) size.
+//this implies to have a correct value for DPI AND that the plplot driver is correctly written.
+void GDLGStream::setVariableCharacterSize( PLFLT charwidthpixel, PLFLT scale , PLFLT lineSpacingpixel, PLFLT xpxcm, PLFLT ypxcm)
 {
-  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"schr(%f,%f,%f)\n",charwidthmm,scale,lineSpacingmm);
-  plstream::schr(charwidthmm, scale);
-  this->setLineSpacing(lineSpacingmm);
-  gdlDefaultCharInitialized=0;
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"setVariableCharacterSize()\n");
+  //tried by comparison of outputs of 
+  // "set_plot,'ps' & !P.multi=[0,2,2]&a=dist(5)&for i=1,4 do begin&s=i*0.7& plot,a,psym=6,syms=s,chars=s,xtit="XXXX" $
+  //  & xyouts,indgen(25),a,"M",ali=0.5,chars=s & end & !p.multi=0 & plots,[0.001,0.001,0.999,0.999,0.001],$
+  //  [0.001,0.999,0.999,0.001,0.001],/norm & device,/close & set_plot,'x'
+
+ if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"schr(width=%f, scale=%f, spacing=%f, xpxcm=%f, ypxcm=%f)\n",charwidthpixel,scale,lineSpacingpixel,xpxcm,ypxcm);
+   PLFLT xdpi=xpxcm*INCHToCM;
+   PLFLT ydpi=ypxcm*INCHToCM;
+// GDL asks for pixels, plplot asks for mm size, but plplot's dpi is always wrong!
+// to get 'charwidthpixel' with this 'wrongdpi', wee need to ask a (wrong) mm size of the character HEIGHT, 
+// BUT we do not know the height/width ratio of the font used, plus the fact that some drivers make the wrong calculation!!!!
+// AND!!! the expected pixel size (X_CH_SIZE) is the true pixel on screen (or paper), not the fake one
+  PLFLT expectedheight_in_mm=charwidthpixel/xdpi*INCHToMM*DEFAULT_FONT_ASPECT_RATIO; //start with a height/width of FONT_ASPECT_RATIO (guessed) will be updated later.
+ if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"current (fake?) dpi is %f : asking for a mm height size: %f)\n",ydpi, expectedheight_in_mm);
+   plstream::schr(expectedheight_in_mm, 1); 
+//trick: if 'em' is not 0, we have the character real width, in mm. It is assumed that when 0, then the size is OK
+//if not 0, then we know the height/width ratio and can recompute the 'good' height that will give the 'good' width (in pixels) 
+   PLFLT em=0;
+#if PLPLOT_PRIVATE_NOT_HIDDEN
+    em=gdlGetStringLength(ALLCHARACTERSFORSTRINGLENGTHTEST)/ALLCHARACTERSFORSTRINGLENGTHTEST_NCHARS; //mean of all
+    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"Able to check character width=%f, should have been %f\n",em, charwidthpixel/xdpi*INCHToMM);
+#endif
+  if (em > 0) {
+    PLFLT ratio=charwidthpixel/xdpi*INCHToMM/em;
+    plstream::schr(expectedheight_in_mm*ratio, 1);
+   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"got plplot character height in mm=%f (2nd pass))\n",pls->chrdef);
+//#if PLPLOT_PRIVATE_NOT_HIDDEN
+//    em=gdlGetStringLength(PATTERN)/PATTERN_LENGTH; //mean of all
+//    ratio=charwidthpixel/xdpi*INCHToMM/em;
+//    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"re-check character width=%f, ratio is %f\n",em, ratio);
+//#endif
+  }
+  setLineSpacing(lineSpacingpixel/ydpi*INCHToMM); //this one is NOT related to characters idiosyncrasies.
+  gdlDefaultCharInitialized=0; //reset Default
+  CurrentCharSize(scale);
+}
+
+
+//This defines the character size for FIXED character devices (X, Z )
+//The dimension of "average" character (given by X_CH_SIZE) is to be a number of pixels on screen
+void GDLGStream::setFixedCharacterSize( PLFLT charwidthpixel, PLFLT scale , PLFLT lineSpacingpixel) {
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"setFixedCharacterSize()\n");
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"schr(width=%f, scale=%f, spacing=%f)\n",charwidthpixel,scale,lineSpacingpixel);
+// GDL asks for pixels, plplot asks for mm size, but plplot's dpi is always wrong!
+// to get 'charwidthpixel' with this 'wrongdpi', wee need to ask a (wrong) mm size of the character HEIGHT, 
+// BUT we do not know the height/width ratio of the font used, plus the fact that some drivers make the wrong calculation!!!!
+// AND!!! the expected pixel size (X_CH_SIZE) is the true pixel on screen (or paper), not the fake one
+  PLFLT expectedheight=charwidthpixel/pls->xdpi*INCHToMM*DEFAULT_FONT_ASPECT_RATIO; //start with a height/width of FONT_ASPECT_RATIO (guessed) will be updated later.
+ if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"current (fake?) dpi is %f : asking for a mm height size: %f)\n",pls->ydpi, expectedheight);
+   plstream::schr(expectedheight, 1); 
+//trick: if 'em' is not 0, we have the character real width, in mm. It is assumed that when 0, then the size is OK
+//if not 0, then we know the height/width ratio and can recompute the 'good' height that will give the 'good' width (in pixels) 
+   PLFLT em=0;
+#if PLPLOT_PRIVATE_NOT_HIDDEN
+    em=gdlGetStringLength(ALLCHARACTERSFORSTRINGLENGTHTEST)/ALLCHARACTERSFORSTRINGLENGTHTEST_NCHARS; //mean of all
+    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"Able to check character width=%f, should have been %f\n",em, charwidthpixel/pls->xdpi*INCHToMM);
+#endif
+  if (em > 0) {
+    PLFLT ratio=charwidthpixel/pls->xdpi*INCHToMM/em;
+    plstream::schr(expectedheight*ratio, 1);
+   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"got plplot character height in mm=%f (2nd pass))\n",pls->chrdef);
+//#if PLPLOT_PRIVATE_NOT_HIDDEN
+//    em=gdlGetStringLength(PATTERN)/PATTERN_LENGTH; //mean of all
+//    ratio=charwidthpixel/pls->xdpi*INCHToMM/em;
+//    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"re-check character width=%f, ratio is %f, in pixels:%f\n",em, ratio, em/INCHToMM*pls->xdpi);
+//#endif
+  }
+ setLineSpacing(lineSpacingpixel/pls->ydpi*INCHToMM); //this one is NOT related to characters idiosyncrasies.
+  gdlDefaultCharInitialized=0; //reset Default
   CurrentCharSize(scale);
 }
 
 void GDLGStream::sizeChar( PLFLT scale )
 {
-    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"SizeChar(%f)\n",scale);
-  plstream::schr(theDefaultChar.mmsx, scale);
-//  plstream::schr(0, scale);
-  CurrentCharSize(scale);
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"SizeChar(%f)\n",scale);
+  plstream::schr(theDefaultChar.mmsx, scale*psCharFudge); //must FORCE a new size.
+  CurrentCharSize(scale*psCharFudge);
 }
 
-void GDLGStream::vpor(PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
+bool GDLGStream::vpor(PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
 {
+  //make vpor units really min max, otherwise som problems appear spuriously
+  // if (xmin > xmax || ymin > ymax) return true; // invalid: see isue #1387
+  if (xmin >= xmax) {xmin=0; xmax=1;} //#1387
+  if (ymin >= ymax) {ymin=0; ymax=1;} //#1387
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"vpor(): requesting x[%f:%f],y[%f:%f] (normalized, subpage)\n",xmin,xmax,ymin,ymax);
   //note that plplot apparently does not write the y=0 line of pixels (in device coords). IDL page is on the contrary limited to
   // [0..1[ in both axes (normalized coordinates)
@@ -960,20 +1240,44 @@ void GDLGStream::vpor(PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
   theBox.nx2=xmax;
   theBox.ny1=ymin;
   theBox.ny2=ymax;
-  PLFLT x1,x2,y1,y2;
-  plstream::gvpd(x1,x2,y1,y2); //retrieve NORMALIZED DEVICE coordinates of viewport
-  theBox.ndx1=x1;
-  theBox.ndx2=x2;
-  theBox.ndy1=y1;
-  theBox.ndy2=y2;
-  theBox.ondx=x1;
-  theBox.ondy=y1;
-  theBox.sndx=x2-x1;
-  theBox.sndy=y2-y1;
-
+  theBox.ndx1=xmin;
+  theBox.ndx2=xmax;
+  theBox.ndy1=ymin;
+  theBox.ndy2=ymax;
+  theBox.nxsize=xmax-xmin;
+  theBox.nysize=ymax-ymin;
   theBox.initialized=true;
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"vpor(): got x[%f:%f],x[%f:%f] (normalized, device)\n",theBox.ndx1,theBox.ndx2,theBox.ndy1,theBox.ndy2);
   syncPageInfo();
+  return false;
+}
+
+//returns true if positioning problem
+bool GDLGStream::isovpor(PLFLT x1, PLFLT x2, PLFLT y1,  PLFLT y2,  PLFLT aspect)
+{
+  if (aspect <= 0.0) {
+    return vpor(x1, x2, y1, y2);
+  }
+  if (x2 <= x1 || y2 <= y1) return true;
+  //x1 < x2 && y1 < y2 implied
+  PLFLT x1mm = nd2mx(x1);
+  PLFLT y1mm = nd2my(y1);
+  PLFLT x2mm = nd2mx(x2);
+  PLFLT y2mm = nd2my(y2);
+  PLFLT ys = y2mm - y1mm; //x and y are in normalized coordinates. ISO scaling must be performed using screen (or paper) coordinates:
+  PLFLT xs = x2mm - x1mm;
+  PLFLT page_aspect=ys/xs;
+  aspect=page_aspect/aspect;
+   if (aspect > 1) { //x ok, resize y
+    y2mm = y1mm + ys/aspect;
+  } else {
+    x2mm = x1mm + xs*aspect;
+  }
+  x1 = mm2ndx(x1mm);
+  x2 = mm2ndx(x2mm);
+  y1 = mm2ndy(y1mm);
+  y2 = mm2ndy(y2mm);
+  return vpor(x1, x2, y1, y2);
 }
 
 void GDLGStream::wind( PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
@@ -989,60 +1293,133 @@ void GDLGStream::wind( PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
   theBox.wy2=ymax;
   updateBoxDeviceCoords();
   UpdateCurrentCharWorldSize();
+  setSymbolSizeConversionFactors(); //because symbols are written in world coordinates. 
 }
 
-void GDLGStream::ssub(PLINT nx, PLINT ny)
+void GDLGStream::wind( PLFLT xmin, PLFLT xmax, bool xLog, PLFLT ymin, PLFLT ymax, bool yLog )
 {
-  plstream::ssub( nx, ny ); // does not appear to change charsize.
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"wind(): setting x[%f:%f],y[%f:%f] (world) \n",xmin,xmax,ymin,ymax);
+  if (xLog) {
+    xmin = log10(xmin);
+    xmax = log10(xmax);
+  }
+  if (yLog) {
+    ymin = log10(ymin);
+    ymax = log10(ymax);
+  }
+  if (xmin==xmax) {xmin=0; xmax=1;}
+  if (ymin==ymax) {ymin=0; ymax=1;}
+  plstream::wind(xmin, xmax, ymin, ymax);
+  theBox.wx1=xmin;
+  theBox.wx2=xmax;
+  theBox.wy1=ymin;
+  theBox.wy2=ymax;
+  updateBoxDeviceCoords();
+  UpdateCurrentCharWorldSize();
+  setSymbolSizeConversionFactors(); //because symbols are written in world coordinates. 
+}
+void GDLGStream::ssub(PLINT nx, PLINT ny, PLINT nz)
+{
+//  plstream::ssub( nx, ny ); // does not appear to change charsize.
+
   // set subpage numbers in X and Y
-  thePage.nbPages=nx*ny;
+  thePage.nbPages=nx*ny*nz;
   thePage.nx=nx;
   thePage.ny=ny;
-  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"ssub() %dx%d pages\n",nx,ny);
+  thePage.nz=nz;
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"ssub() %dx%dx%d pages\n",nx,ny,nz);
   thePage.curPage=1;
   syncPageInfo();
 }
 
 void GDLGStream::adv(PLINT page)
 {
-  plstream::adv(page);
+//  plstream::adv(page); //plstream below soes not advance pages as it has only ONE.
   if (page==0) {thePage.curPage++;} else {thePage.curPage=page;}
   if (thePage.curPage > thePage.nbPages) thePage.curPage=1;
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"adv() now at page %d\n",thePage.curPage);
+  PLFLT sxmin,symin,sxmax,symax,szmin,szmax;
+  getSubpageRegion(&sxmin,&symin,&sxmax,&symax,&szmin,&szmax);
+  //SET ALL REGION TAGS
+  unsigned regionTag=SysVar::X()->Desc()->TagIndex("REGION");
+  (*static_cast<DFloatGDL*>(SysVar::X()->GetTag(regionTag, 0)))[0]=sxmin;
+  (*static_cast<DFloatGDL*>(SysVar::X()->GetTag(regionTag, 0)))[1]=sxmax;
+  regionTag=SysVar::Y()->Desc()->TagIndex("REGION");
+  (*static_cast<DFloatGDL*>(SysVar::Y()->GetTag(regionTag, 0)))[0]=symin;
+  (*static_cast<DFloatGDL*>(SysVar::Y()->GetTag(regionTag, 0)))[1]=symax;
+  regionTag=SysVar::Z()->Desc()->TagIndex("REGION");
+  (*static_cast<DFloatGDL*>(SysVar::Z()->GetTag(regionTag, 0)))[0]=szmin;
+  (*static_cast<DFloatGDL*>(SysVar::Z()->GetTag(regionTag, 0)))[1]=szmax;
+}
+
+void GDLGStream::getSubpageRegion(PLFLT *sxmin, PLFLT *symin, PLFLT *sxmax, PLFLT *symax, PLFLT *szmin, PLFLT *szmax) {
+  //here we must take into account the contents of ![X|Y|Z].OMARGIN
+  unsigned int omarginTag = SysVar::X()->Desc()->TagIndex("OMARGIN");
+  DFloat xstart = (*static_cast<DFloatGDL*> (SysVar::X()->GetTag(omarginTag, 0)))[0];
+  xstart=MAX(xstart,0);
+  DFloat xend = (*static_cast<DFloatGDL*> (SysVar::X()->GetTag(omarginTag, 0)))[1];
+  xend=MAX(xend,0);
+  omarginTag = SysVar::Y()->Desc()->TagIndex("OMARGIN");
+  DFloat ystart = (*static_cast<DFloatGDL*> (SysVar::Y()->GetTag(omarginTag, 0)))[0];
+  ystart=MAX(ystart,0);
+  DFloat yend = (*static_cast<DFloatGDL*> (SysVar::Y()->GetTag(omarginTag, 0)))[1];
+  yend=MAX(yend,0);
+  //  Z OMARGIN to BE CHECKED and code must be written.
+  //  omarginTag = SysVar::Z()->Desc()->TagIndex("OMARGIN");
+  //  DFloat zstart = (*static_cast<DFloatGDL*> (SysVar::Z()->GetTag(omarginTag, 0)))[0];
+  //  DFloat zend = (*static_cast<DFloatGDL*> (SysVar::Z()->GetTag(omarginTag, 0)))[1];
+  DFloat xNormedPageSize=1-(xend+xstart)*theCurrentChar.ndsx;
+  DFloat yNormedPageSize=1-(yend+ystart)*theCurrentChar.nspacing;
+  if (xNormedPageSize < 0 || xNormedPageSize > 1 || yNormedPageSize < 0 || yNormedPageSize > 1) {
+	Message("Data coordinate system not established.");
+	if (xNormedPageSize < 0) xNormedPageSize=0;
+	if (yNormedPageSize < 0) yNormedPageSize=0;
+	if (xNormedPageSize > 1) xNormedPageSize=1;
+	if (yNormedPageSize > 1) yNormedPageSize=1;
+  } 
+  DFloat zNormedPageSize=1; //zend-zstart??;
+  DFloat xNormedOffset=xstart*theCurrentChar.ndsx;
+  DFloat yNormedOffset=yend*theCurrentChar.nspacing;
+  //check silly values: normed must be >0 and < 1
+  int p=thePage.curPage-1;
+  PLFLT width=xNormedPageSize/thePage.nx;
+  PLFLT height=yNormedPageSize/thePage.ny;
+  PLFLT profund=zNormedPageSize/thePage.nz;
+ int k= p / (thePage.nx*thePage.ny);
+ int l= p - k*(thePage.nx*thePage.ny);
+ int j= l /thePage.nx ;
+ int i= (l - j*thePage.nx);
+ *sxmin=i*width+xNormedOffset;
+ *sxmax=*sxmin+width;
+ *symax=1-(j*height+yNormedOffset);
+ *symin=*symax-height;
+ if (szmin != NULL) {
+   *szmin=k*profund;
+   *szmax=*szmin+profund;
+ }
 }
 //get region (3BPP data)
-bool GDLGStream::GetRegion(DLong& x_gdl, DLong& y_gdl, DLong& nx_gdl, DLong& ny_gdl){
-    DByteGDL *bitmap = static_cast<DByteGDL*>(this->GetBitmapData());
-    if (bitmap==NULL)  return false; //need to GDLDelete bitmap on exit after this line.
 
-    bool error=false;
-    DLong nx=bitmap->Dim(0);
-    DLong ny=bitmap->Dim(1);
-    
-    DLong xref,xval,xinc,yref,yval,yinc,xmax11,ymin11;
-    long x_11=0;
-    long y_11=0;
-    xref=0;xval=0;xinc=1;
-    yref=0;yval=0;yinc=1;
-    
-    x_11=xval+(x_gdl-xref)*xinc;
-    y_11=yval+(y_gdl-yref)*yinc;
-    xmax11=xval+(x_gdl+nx_gdl-1-xref)*xinc;    
-    ymin11=yval+(y_gdl+ny_gdl-1-yref)*yinc;
-    if (y_11 < 0 || y_11 > ny-1) error=true;
-    if (x_11 < 0 || x_11 > nx-1) error=true;
-    if (xmax11 < 0 || xmax11 > nx-1) error=true;
-    if (ymin11 < 0 || ymin11 > ny-1) error=true;
-    if (error) {  GDLDelete(bitmap); return false; }
-    GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
-    unsigned char* data=actDevice->SetCopyBuffer(nx_gdl*ny_gdl*3);  
-    for ( SizeT i =0; i < nx_gdl ; ++i ) {
-      for ( SizeT j = 0; j < ny_gdl ; ++j ) {
-       for ( SizeT k = 0 ; k < 3 ; ++k) data[3 * (j * nx_gdl + i) + k] = (*bitmap)[3 * ((j+y_11) * nx + (i+x_11)) + k]; 
-      }
-    }
-    GDLDelete(bitmap);
-    return true;
+bool GDLGStream::GetRegion(DLong& xoff, DLong& yoff, DLong& nx, DLong& ny) {
+  long nxOrig,nyOrig;
+  this->GetGeometry(nxOrig,nyOrig);
+
+  DLong xmax = xoff + nx - 1;
+  DLong ymax = yoff + ny - 1;
+  if (yoff < 0 || yoff > nyOrig - 1) return false;
+  if (xoff < 0 || xoff > nxOrig - 1) return false;
+  if (xmax < 0 || xmax > nxOrig - 1) return false;
+  if (ymax < 0 || ymax > nyOrig - 1) return false;
+
+  DByteGDL *bitmap = static_cast<DByteGDL*> (this->GetBitmapData(xoff,yoff,nx,ny));
+  if (bitmap == NULL) return false; //need to GDLDelete bitmap on exit after this line.
+  DByte* bmp=static_cast<DByte*>(bitmap->DataAddr());
+
+  GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
+  unsigned char* data = actDevice->SetCopyBuffer(nx * ny * 3);
+  for (auto k=0; k< nx*ny*3; ++k) data[k] = bmp[k];
+  GDLDelete(bitmap);
+  return true;
 }
 
 bool GDLGStream::SetRegion(DLong& xs, DLong& ys, DLong& nx, DLong& ny){
@@ -1050,3 +1427,6 @@ bool GDLGStream::SetRegion(DLong& xs, DLong& ys, DLong& nx, DLong& ny){
   GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
   return this->PaintImage(actDevice->GetCopyBuffer(), nx, ny, pos, 1, 0);  
 }
+//
+////plplot main functions patched for 3d
+//#include "plplotdriver/plutils.h"
