@@ -25,12 +25,32 @@
 
 // use "plain menubars" instead of 'taskbars used as menubars'. taskbars permit to change font in bar, put pixmaps instead of text, and will work
 // on OSX. So we choose normally to undefine this 
-// #define PREFERS_MENUBAR 1
+#ifndef __WXMAC__ 
+//warning MAC should not have prefer_menubar=1 unless you solve the mac manubar problem.
+#define PREFERS_MENUBAR 1
+#endif
+// For compilers that support precompilation, includes "wx/wx.h".
+// HAVE_LARGEFILE_SUPPORT, SIZEOF_VOID_P, SIZEOF_SIZE_T,  may be set by Python, creates unnecessary warnings
+#ifdef HAVE_LARGEFILE_SUPPORT
+#undef HAVE_LARGEFILE_SUPPORT
+#endif
+#ifdef SIZEOF_VOID_P
+#undef SIZEOF_VOID_P
+#endif
+#ifdef SIZEOF_SIZE_T
+#undef SIZEOF_SIZE_T
+#endif
 #include <wx/wx.h>
+
 #include <wx/app.h>
 #include <wx/panel.h>
 #include <wx/treebase.h>
+
 #include <wx/treectrl.h>
+#include <wx/dragimag.h>
+#include <wx/dcbuffer.h>
+#include <wx/dnd.h>
+
 #include <wx/grid.h>
 #ifdef HAVE_WXWIDGETS_PROPERTYGRID
 //#include <wx/propgrid/propgrid.h>
@@ -41,6 +61,9 @@
 #include <wx/artprov.h>
 #include <wx/popupwin.h>
 #include <wx/notebook.h>
+#include <wx/toolbook.h>
+ #define gdlNotebook wxNotebook
+//#define gdlNotebook wxToolbook
 #include <wx/dcbuffer.h>
 #include <wx/toolbar.h>
 #include <wx/listbox.h>
@@ -52,10 +75,10 @@
 #include "datatypes.hpp"
 #include "widget.hpp"
 
-#define gdlSCROLL_RATE 20
-#define gdlSCROLL_HEIGHT_X  sysScrollHeight //wxSystemSettings::GetMetric(wxSYS_VSCROLL_X,xxx) //25 
-#define gdlSCROLL_WIDTH_Y sysScrollWidth //wxSystemSettings::GetMetric(wxSYS_HSCROLL_Y,xxx) //25
-#define gdlABSENT_SIZE_VALUE 15; 
+#define gdlSCROLL_RATE 10
+#define gdlABSENT_SIZE_VALUE 15
+#define gdlSCROLL_HEIGHT_X  ((sysScrollHeight < 10) ? gdlABSENT_SIZE_VALUE: sysScrollHeight) //wxSystemSettings::GetMetric(wxSYS_VSCROLL_X,xxx) //25 
+#define gdlSCROLL_WIDTH_Y ((sysScrollWidth< 10) ? gdlABSENT_SIZE_VALUE: sysScrollWidth) //wxSystemSettings::GetMetric(wxSYS_HSCROLL_Y,xxx) //25
 #define gdlDEFAULT_XSIZE 100
 #define gdlDEFAULT_YSIZE 100
 #define gdlCOMBOBOX_ARROW_WIDTH sysComboboxArrow 
@@ -68,13 +91,13 @@
 #define gdlBUTTON_SPACE 4
 #define gdlSMALL_SPACE 1
 #define gdlBORDER_SPACE 2
-#define gdlBORDER_EXT wxBORDER_THEME // wxBORDER_SUNKEN //wxBORDER_RAISED//wxBORDER_SIMPLE //wxBORDER_RAISED
-#define gdlBORDER_INT wxBORDER_NONE //wxBORDER_SUNKEN 
+#define gdlBORDER_EXT  wxBORDER_SUNKEN // wxBORDER_THEME // wxBORDER_SUNKEN //wxBORDER_RAISED//wxBORDER_SIMPLE //wxBORDER_RAISED
+#define gdlBORDER_INT wxBORDER_RAISED //wxBORDER_SUNKEN 
 #define gdlTEXT_XMARGIN 4
 #define gdlTEXT_YMARGIN 4
 #define DONOTALLOWSTRETCH 0
-#define ALLOWSTRETCH 1
-#define FRAME_ALLOWSTRETCH 1
+#define ALLOWSTRETCH 0
+#define FRAME_ALLOWSTRETCH 0
 #define DEFAULT_TREE_IMAGE_SIZE 16
 
 #ifdef __WXMSW__
@@ -82,8 +105,11 @@
 #else
   #define NEWLINECHARSIZE 1  //length of <nl> 
 #endif
-
+#ifdef __WXMSW__
+#define gdlSIZE_EVENT_HANDLER wxSizeEventHandler(gdlwxFrame::OnSize) //Timer resize do not work on MSW
+#else
 #define gdlSIZE_EVENT_HANDLER wxSizeEventHandler(gdlwxFrame::OnSizeWithTimer) //filter mouse events (manual resize) to avoid too many updtes for nothing
+#endif
 #define gdlSIZE_IMMEDIATE_EVENT_HANDLER wxSizeEventHandler(gdlwxFrame::OnSize) 
 typedef DLong WidgetIDT;
 static std::string widgetNameList[]={"BASE","BUTTON","SLIDER","TEXT","DRAW","LABEL","LIST","MBAR","DROPLIST","TABLE","TAB","TREE","COMBOBOX","PROPERTYSHEET","WINDOW"};
@@ -216,22 +242,15 @@ public:
 };
 #ifndef __WXMAC__
 // main App class
+ #include "wx/evtloop.h"
+ 
 class wxAppGDL: public wxApp
 {
- wxEventLoopBase* loop;
+ wxGUIEventLoop loop;
 public:
- int OnRun(){return 0;}
- int OnExit();
- int MainLoop();
-// virtual int OneLoop();
- bool OnInit();
-// bool Pending(); //Returns true if unprocessed events are in the window system event queue.
-// int FilterEvent(wxEvent& event) //This function is called before processing any event and 
-//allows the application to preempt the processing of some events. If this method returns -1
-//the event is processed normally, otherwise either true or false should be returned and 
-//the event processing stops immediately considering that the event had been already processed
-//(for the former return value) or that it is not going to be processed at all (for the latter one).
+ int MyLoop();
 };
+
 wxDECLARE_APP(wxAppGDL); //wxAppGDL is equivalent to wxGetApp()
 #endif
 
@@ -251,17 +270,22 @@ class gdlwxFrame : public wxFrame {
  wxSize frameSize;
  wxTimer * m_resizeTimer;
  GDLWidgetTopBase* gdlOwner;
-
+ wxWindowDisabler *m_windowDisabler; //for modal bases
 public:
 
  // ctor(s)
- gdlwxFrame(wxWindow* parent, GDLWidgetTopBase* gdlOwner_, wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style=wxDEFAULT_FRAME_STYLE);
+ gdlwxFrame(wxWindow* parent, GDLWidgetTopBase* gdlOwner_, wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style=wxDEFAULT_FRAME_STYLE, bool modal=false);
  ~gdlwxFrame();
  // called from ~GDLWidgetBase
  void NullGDLOwner() {
   gdlOwner = NULL;
  }
-
+ void UnblockIfModal() {
+    if (m_windowDisabler != NULL) {
+      delete m_windowDisabler; //destructor should reenable all windows.
+      m_windowDisabler = NULL;
+    }
+ }
  GDLWidgetTopBase* GetGDLOwner() {
   return gdlOwner;
  }
@@ -313,7 +337,7 @@ public:
  void OnTextMouseEvents(wxMouseEvent& event);
  void OnTextPaste(wxClipboardTextEvent& event);
  void OnTextCut(wxClipboardTextEvent& event);
- //  void OnTextEnter( wxCommandEvent& event); //NOT USED
+ void OnTextEnter( wxCommandEvent& event); //NOT USED
  void OnThumbTrack(wxScrollEvent& event);
  void OnThumbRelease(wxScrollEvent& event);
  void OnSize(wxSizeEvent& event); //unused.
@@ -334,6 +358,7 @@ public:
  void OnTracking(wxFocusEvent& event);
  void OnWidgetTimer(wxTimerEvent & event);
  void OnKBRDFocusChange(wxFocusEvent &event);
+void OnDrag(wxMouseEvent& event);
 private:
  void OnListBoxDo(wxCommandEvent& event, DLong clicks);
  DECLARE_EVENT_TABLE()
@@ -342,7 +367,7 @@ private:
 static int sysScrollHeight=25;
 static int sysScrollWidth=25;
 static int sysComboboxArrow=25;
-  
+static wxUint32 sysPanelDefaultColour;  
 class GDLWidget
 { 
   // static part is used for the abstraction
@@ -358,12 +383,12 @@ private:
 public:
   static wxFont defaultFont;
   static wxFont systemFont;
-  static GDLEventQueue eventQueue;
-  static GDLEventQueue readlineEventQueue;
+  static GDLEventQueue BlockingEventQueue;
+  static GDLEventQueue InteractiveEventQueue;
   static void PushEvent( WidgetIDT baseWidgetID, DStructGDL* ev);
   static void InformAuthorities(const std::string& message);
   
-  static void HandleWidgetEvents();
+  static void HandleUnblockedWidgetEvents();
   static const WidgetIDT NullID;
   
   GDLWidget( WidgetIDT p, EnvT* e, BaseGDL* vV=NULL, DULong eventFlags_=0);
@@ -391,6 +416,7 @@ public:
   static bool InitWx(); // global start of wxWidgets
   static void Init(); // global GUI intialization upon GDL startup
   static void UnInit(); // global GUI desinitialization in case it is useful (?)
+  static void ResetWidgets(); // for widget_control,/reset and UnInit
   static bool wxIsStarted(){return (wxIsOn);}
   static void SetWxStarted(){wxIsOn=true;}
   static void UnsetWxStarted(){gdl_lastControlId=0;/* not possible: wxWidgets library does not survive wxUniitiailze() ... wxIsOn=false; handlersOk=false;*/}
@@ -403,7 +429,13 @@ public:
    defaultFont=thefont;
   }
   static BaseGDL * getSystemColours();
-
+  static void CallWXEventLoop(){
+#ifdef __WXMAC__
+    wxTheApp->Yield();
+#else
+    wxGetApp().MyLoop(); //central loop for wxEvents!
+#endif
+  }
 protected:
   
 
@@ -441,13 +473,13 @@ protected:
   wxFont       font;
   bool         valid; //if not, is in the process of being destroyed (prevent reentrance).
   long  alignment; //alignment of the widget
-  int widgetStyle; //style (alignment code + other specific codes used as option to widgetsizer). Needed only because of frame/unframe  function
   int dynamicResize; //for some widgets, will enable resizing: -1: not resizable, 0/1 resizable
   std::vector<WidgetIDT> followers; //all the widgets that use me as group_leader
   std::vector<WidgetEventInfo*> desiredEventsList; //list of all the events (and handlers) this widget must obey.
   DString      notifyRealize;
   
   wxTimer * m_windowTimer;
+//  bool delay_destroy;
   
 private:  
 
@@ -463,6 +495,8 @@ private:
   
 public:
 
+//  void SetDelayDestroy(bool val){delay_destroy=val;}
+//  bool GetDelayDestroy(){return delay_destroy;}
  void setFont();
  void setFont(wxObject* o);
 
@@ -587,8 +621,6 @@ public:
   virtual void SetWidgetVirtualSize(DLong sizex, DLong sizey){}; //do Nothing
   virtual void SetWidgetScreenSize(DLong sizex, DLong sizey);
   void SetWidgetPosition(DLong posx, DLong posy);
-  DLong GetXPos(){return dynamic_cast<wxWindow*>(theWxWidget)->GetPosition().x;}
-  DLong GetYPos(){return dynamic_cast<wxWindow*>(theWxWidget)->GetPosition().y;}
   bool IsValid(){return valid;}
   void SetUnValid(){valid=false;}
   void SetValid(){valid=true;}
@@ -611,6 +643,8 @@ public:
   virtual bool IsContainer() const { return false;} 
   virtual bool IsBase() const { return false;} 
   virtual bool IsTopBase() const { return false;} 
+  virtual bool IsNormalBase() const { return false;} 
+  virtual bool IsTabbedBase() const { return false;} 
   virtual bool IsContextBase() const {  return false; }
   virtual bool IsButton() const { return false;} 
   virtual bool IsMenu() const { return false;}
@@ -638,9 +672,9 @@ public:
   //returns a list of IDs of all the widgets starting at me and below.
   DLongGDL* GetAllHeirs();
   
-  virtual void SetXmanagerActiveCommand() {std::cerr<<"XMANAGER ACTIVE COMMAND on a not-top widget, please report."<<std::endl;}
+  virtual void MakeInteractive() {std::cerr<<"XMANAGER ACTIVE COMMAND on a not-top widget, please report."<<std::endl;}
   
-  bool GetXmanagerActiveCommand();
+  virtual bool IsUsingInteractiveEventLoop() {/*std::cerr<<"IsEventLoopBlocked on a not-top widget, please report."<<std::endl*/;return false;} //default for a normal widget
 
   void SetEventPro( const DString& ePro) { eventPro = StrUpCase( ePro);}
   const DString& GetEventPro() const { return eventPro;};
@@ -651,7 +685,8 @@ public:
   void SetKillNotify( const DString& eKN) { killNotify = StrUpCase( eKN);}
   const DString& GetKillNotify() const { return killNotify;}
 
-  static bool GetXmanagerBlock();
+  static bool IsXmanagerBlocking();
+  static bool IsActive();
   static DLong GetNumberOfWidgets();
   static BaseGDL* GetWidgetsList();
   static BaseGDL* GetManagedWidgetsList();
@@ -812,8 +847,6 @@ public:
   void CreateBase(wxWindow* parent);
   bool IsVertical() {return (ncols>0);}
   bool IsHorizontal() {return (nrows>0);}
-  virtual bool IsNormalBase() const { return false;} 
-  virtual bool IsTabbedBase() const { return false;} 
   void SetWidgetSize(DLong sizex, DLong sizey) final;
  
   void NullWxWidget() { theWxWidget = NULL;}
@@ -846,14 +879,15 @@ public:
    GDLWidget* w = GetWidget(*c);
    if (w != NULL && w->IsBase()) static_cast<GDLWidgetBase*>(w)->DoMapAsRequested();
   }
-  dynamic_cast<wxWindow*>(theWxContainer)->Show(doMap);
+  mapBase(doMap);
  }
+ virtual void SetButtonWidget( bool onOff) final;
 };
 
 class GDLWidgetTopBase : public GDLWidgetBase {
  WidgetIDT mbarID;
 public:
- bool xmanActCom; //set by /XMANAGER_ACTIVE_COMMAND (GDL's) aka NO_BLOCK . indirectly used in pushEvent, selfDestroy, ~GDLWidgetContainer, ClearEvents
+ bool UseInteractiveEvents; //set by /XMANAGER_ACTIVE_COMMAND (GDL's) aka NO_BLOCK . indirectly used in pushEvent, selfDestroy, ~GDLWidgetContainer, ClearEvents
  gdlwxFrame* topFrame;
  bool modal;
  bool realized;
@@ -901,9 +935,11 @@ public:
    if (w != NULL)
     w->OnRealize();
   }
-  topFrame->Fit();
-  topFrame->refreshFrameSize();
-  DoMapAsRequested();
+
+    topFrame->Fit();
+    topFrame->SetClientSize(topFrame->GetClientSize());
+    topFrame->refreshFrameSize();
+
   ConnectToDesiredEvents();
   
   if (notifyRealize != "") { //insure it is called once only for this.
@@ -911,16 +947,23 @@ public:
    notifyRealize.clear();
    CallEventPro(note, new DLongGDL(widgetID));
   }
+  DoMapAsRequested();
  }
 
 // void SelfDestroy(); // sends delete event to itself
 
- void SetXmanagerActiveCommand() {
-  xmanActCom = true;
+ virtual void MakeInteractive() final {
+#ifdef GDL_DEBUG_WIDGETS
+          wxMessageOutputStderr().Printf(_T("MakeInteractive(%d)\n"),widgetID);
+#endif
+  UseInteractiveEvents = true; //unblocks base
  }
-
- bool GetXmanagerActiveCommand() const {
-  return xmanActCom;
+// returns UseInteractiveEvents
+ virtual bool IsUsingInteractiveEventLoop() final {
+#ifdef GDL_DEBUG_WIDGETS
+          wxMessageOutputStderr().Printf(_T("IsUsingInteractiveEventLoop(%d): %d\n"),widgetID,UseInteractiveEvents);
+#endif
+  return UseInteractiveEvents;  //false (blocked) by default
  }
  
 };
@@ -971,7 +1014,7 @@ public:
  
  ~GDLWidgetTabbedBase();
  void SetBaseTitle(std::string &s);
- bool IsTabbedBase() const {  return true; }
+ bool IsTabbedBase() const final { return true;} 
  void mapBase(bool val) final;
 
  //Same as Container except that we have to reorder widgets in some cases
@@ -1038,7 +1081,7 @@ public:
  typedef enum ButtonType_ {
   UNDEFINED=-1, NORMAL=0, RADIO=1, CHECKBOX=2, MENU=3, ENTRY=4, BITMAP=5, POPUP_NORMAL=6, POPUP_BITMAP=7} ButtonType;
  ButtonType buttonType;
-  wxBitmap* buttonBitmap;
+//  wxBitmap* buttonBitmap;
   wxMenuItem* menuItem;
   bool       buttonState; //only for buttons
   wxString valueWxString;
@@ -1264,7 +1307,7 @@ public:
   
 //   void SetSelectOff();
   bool IsComboBox() const final { return true;} 
-
+  wxSize computeWidgetSize();
   void SetLastValue( const std::string& v) { lastValue = v;}
   std::string GetLastValue() { return lastValue;}
   void SetValue(BaseGDL *value);
@@ -1358,6 +1401,7 @@ public:
   void SetLabelValue( const DString& value_);
   bool IsLabel() const final { return true;} 
   wxSize computeWidgetSize() final;
+  wxSize updateDynamicWidgetSize();
   int widgetAlignment();
   void SetWidgetSize(DLong sizex, DLong sizey) final; 
 };
@@ -1381,6 +1425,8 @@ public:
   void SetWidgetScreenSize(DLong sizex, DLong sizey) final;
   void UnrefTheWxContainer(){theWxContainer=NULL;} 
   void UnrefTheWxWidget(){theWxWidget=NULL;} 
+  wxPoint GetPos();
+  void SetPos(int x, int y);
 };
 
 // menubar is best done with a toolbar at the moment, see below
@@ -1514,6 +1560,7 @@ public:
   BaseGDL* GetTabCurrent();
   void SetTabCurrent(int val);
   BaseGDL* GetTabMultiline(); //not exactly what expected, fixme.
+  void OnRealize();
 };
 
 
@@ -1540,8 +1587,8 @@ class GDLWidgetTable: public GDLWidget
   DLongGDL* rowHeights;
   DStringGDL* rowLabels;
 //  DLong tabMode;
-  DLong x_scroll_size;
-  DLong y_scroll_size;
+  DLong x_scroll_size_columns;
+  DLong y_scroll_size_rows;
   DStringGDL * valueAsStrings;
   bool         updating; //widget is modified by program (avoid sending events)
 
@@ -1645,14 +1692,51 @@ public:
   void setFont();
 };
 
-
+static unsigned char tree_up_selection_mask_uc[] = {
+  0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x18, 0x00, 0xfe, 0x01, 0x18, 0x01,
+  0x10, 0x01, 0x00, 0x01, 0x00, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+};
+static unsigned char tree_up_selection_bits_uc[] = {
+  0xff, 0xff, 0xff, 0xff, 0xef, 0xff, 0xe7, 0xff, 0x01, 0xfe, 0xe7, 0xfe,
+  0xef, 0xfe, 0xff, 0xfe, 0xff, 0x80, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
+  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
+};
+static unsigned char tree_down_selection_mask_uc[] = {
+   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+   0x00, 0x00, 0x00, 0x7f, 0x00, 0x01, 0x10, 0x01, 0x18, 0x01, 0xfe, 0x01,
+   0x18, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00 };
+static unsigned char tree_down_selection_bits_uc[] = {
+   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
+   0xff, 0xff, 0xff, 0x80, 0xff, 0xfe, 0xef, 0xfe, 0xe7, 0xfe, 0x01, 0xfe,
+   0xe7, 0xff, 0xef, 0xff, 0xff, 0xff, 0xff, 0xff };
+static const char* tree_up_selection_bits = (const char*) tree_up_selection_bits_uc;
+static const char* tree_up_selection_mask = (const char*) tree_up_selection_mask_uc;
+static const char* tree_down_selection_bits = (const char*) tree_down_selection_bits_uc;
+static const char* tree_down_selection_mask = (const char*) tree_down_selection_mask_uc;
 // tree widget **************************************************
+class GDLWidgetTree;
+class wxTreeItemDataGDL;
 class wxTreeCtrlGDL: public wxTreeCtrl {
   wxWindowID GDLWidgetTreeID;
-  wxWindowID draggedGDLWidgetID;
   int KeyModifier; //Shift, Control... set during a drag
-public:
+private:
+	// the item being dragged at the moment
+	wxTreeItemId itemDragging = nullptr;
+  wxCursor gdlTREE_SELECT_ABOVE;
+  wxCursor gdlTREE_SELECT_BELOW;
+  int posCode;
+  wxPoint position;
 
+public:
+//	void onLeftDown(wxMouseEvent &evt);
+	void onLeftUp(wxMouseEvent &evt);
+	void onLeaveWindow(wxMouseEvent &evt);
+	void onEnterWindow(wxMouseEvent &evt);
+	// finish the drag action 
+	void endDragging();
+	void onMouseMotion(wxMouseEvent &evt);
+public:
   wxTreeCtrlGDL(wxWindow *parent, wxWindowID id = wxID_ANY,
     const wxPoint& pos = wxDefaultPosition,
     const wxSize& size = wxDefaultSize,
@@ -1661,44 +1745,44 @@ public:
     const wxString& name = wxTreeCtrlNameStr)
           :wxTreeCtrl( parent, id, pos, size, style, wxDefaultValidator , name ),
           GDLWidgetTreeID(id),
-          draggedGDLWidgetID(0),
-          KeyModifier(0)
+          posCode(-1)
+          , position(wxDefaultPosition)
           {
-//            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_ACTIVATED, wxTreeEventHandler(wxTreeCtrlGDL::OnItemActivated));
-//            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_ACTIVATED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemActivated));
-//            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_BEGIN_DRAG,wxTreeEventHandler(wxTreeCtrlGDL::OnBeginDrag));
-//            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_END_DRAG,wxTreeEventHandler(wxTreeCtrlGDL::OnItemDropped));
-//            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_COLLAPSED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemCollapsed));
-//            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_EXPANDED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemExpanded));
-//            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_SEL_CHANGED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemSelected));
-  }
+
+#ifdef __WXMSW__
+    wxBitmap tree_up_selection_bitmap(tree_up_selection_bits, 16, 16);
+    wxBitmap tree_up_selection_mask_bitmap(tree_up_selection_mask, 16, 16);
+    tree_up_selection_bitmap.SetMask(new wxMask(tree_up_selection_mask_bitmap));
+    wxImage tree_up_selection_image = tree_up_selection_bitmap.ConvertToImage();
+    tree_up_selection_image.SetOption(wxIMAGE_OPTION_CUR_HOTSPOT_X, 14);
+    tree_up_selection_image.SetOption(wxIMAGE_OPTION_CUR_HOTSPOT_Y, 7);
+    gdlTREE_SELECT_ABOVE = wxCursor(tree_up_selection_image);
+    wxBitmap tree_down_selection_bitmap(tree_down_selection_bits, 16, 16);
+    wxBitmap tree_down_selection_mask_bitmap(tree_down_selection_mask, 16, 16);
+    tree_down_selection_bitmap.SetMask(new wxMask(tree_down_selection_mask_bitmap));
+    wxImage tree_down_selection_image = tree_down_selection_bitmap.ConvertToImage();
+    tree_down_selection_image.SetOption(wxIMAGE_OPTION_CUR_HOTSPOT_X, 14);
+    tree_down_selection_image.SetOption(wxIMAGE_OPTION_CUR_HOTSPOT_Y, 7);
+    gdlTREE_SELECT_BELOW = wxCursor(tree_down_selection_image);
+
+#elif defined(__WXGTK__) or defined(__WXMOTIF__)
+    gdlTREE_SELECT_ABOVE = wxCursor(tree_up_selection_bits, 16, 16, 14, 7, tree_up_selection_mask, wxWHITE, wxBLACK);
+    gdlTREE_SELECT_BELOW = wxCursor(tree_down_selection_bits, 16, 16, 14, 7, tree_down_selection_mask, wxWHITE, wxBLACK);
+#endif
+          }
   //necessary to define the destructor otherwise compiler will try to find the bind event table for destruction event!
   ~wxTreeCtrlGDL(){}
-  void SetCurrentModifier(int x, bool remove){ //We have to remap the wxWidgets events
-    switch(x){
-      case WXK_SHIFT:
-        if (remove) KeyModifier &= ~(1); else KeyModifier |= 1; break; //bit 1 
-      case WXK_ALT:
-        if (remove) KeyModifier &= ~(1<<3); else KeyModifier |= (1<<3); break; //bit 4
-      case WXK_NUMLOCK:
-        if (remove) KeyModifier &= ~(1<<2); else KeyModifier |= (1<<2); break; //bit 3
-      case WXK_CONTROL:
-        if (remove) KeyModifier &= ~(1<<1); else KeyModifier |= (1<<1); break; //bit 2
-        
-    }
-  }
   int GetCurrentModifier(){return KeyModifier;}
-  void OnTreeKeyDown(wxKeyEvent & event);
-  void OnTreeKeyUp(wxKeyEvent & event);
+  GDLWidgetTree* GetItemTreeWidget(wxTreeItemId id);
+  
   void OnItemActivated(wxTreeEvent & event);
   void OnItemCollapsed(wxTreeEvent & event);
   void OnItemStateClick(wxTreeEvent & event);
   void OnItemExpanded(wxTreeEvent & event);
   void OnDrag(wxTreeEvent & event);
-  void OnDrop(wxTreeEvent & event);
+  void OnTreeCtrlDrop(wxTreeEvent & event);
+  void OnForeignDrop(WidgetIDT selected, WidgetIDT drag_id);
   void OnItemSelected(wxTreeEvent & event);
-  void SetDragged(wxWindowID that) {draggedGDLWidgetID=that;}
-  wxWindowID GetDragged(){ return draggedGDLWidgetID;}
   DECLARE_EVENT_TABLE()
 };
 
@@ -1706,10 +1790,11 @@ class wxTreeItemDataGDL : public wxTreeItemData {
   public:
     WidgetIDT widgetID;
     wxTreeCtrlGDL* myTree;
-
+    wxTreeItemId treeItemID;
   wxTreeItemDataGDL(WidgetIDT id, wxTreeCtrlGDL* myTree_) : widgetID(id), myTree(myTree_) {}
   WidgetIDT GetWidgetID(){return widgetID;}
   wxTreeCtrlGDL* GetTree(){return myTree;}
+  void SetItemId(wxTreeItemId id){treeItemID=id;}
 };
 
 class GDLWidgetTree: public GDLWidget
@@ -1746,8 +1831,9 @@ GDLWidgetTree( WidgetIDT p, EnvT* e, BaseGDL* value_, DULong eventFlags
   bool GetDragability();
   void SetDragability(DLong val){draggable=val;}
   //DRAGNOTIFY
-  DString GetDragNotifyValue(){return dragNotify;}
-  void SetDragNotify(DString &s){Warning("SET_DRAG_NOTIFY is currently ignored by GDL, FIXME."); dragNotify=s;}
+  DString GetDragNotifyValue();
+  int GetDragNotifyReturn(DString &cf, WidgetIDT sourceID, int modifiers, int defaultval);
+  void SetDragNotify(DString &s){dragNotify=s;}
   //DROP
   int  GetDroppableValue(){return droppable;}
   bool GetDropability();
@@ -1788,7 +1874,7 @@ GDLWidgetTree( WidgetIDT p, EnvT* e, BaseGDL* value_, DULong eventFlags
   DLongGDL* GetChildrenList() const;
   DLong GetTheSiblingOf(DLong myIx) final;
   DLong Sibling();
- };
+};
 
 
 
@@ -2007,7 +2093,6 @@ class gdlwxPhantomFrame : public wxFrame {
 // basic for (completely separated from widgets) plot windows done with wxWidgets driver.
 class gdlwxPlotFrame : public wxFrame {
  wxTimer * m_resizeTimer;
- wxSize frameSize;
  bool scrolled;
 public:
  // ctor(s)
@@ -2016,12 +2101,10 @@ public:
   bool IsScrolled(){return scrolled;}
  void Realize();
 // event handlers (these functions should _not_ be virtual)
-// void OnClosePlotFrame(wxCloseEvent & event);
-// void OnPlotSizeWithTimer(wxSizeEvent& event);
  void OnTimerPlotResize(wxTimerEvent& event);
  void OnUnhandledClosePlotFrame(wxCloseEvent & event);
  void OnPlotSizeWithTimer(wxSizeEvent& event);
- void OnPlotWindowSize(wxSizeEvent& event);
+ void OnPlotWindowSize(wxSizeEvent& event); //unused
  DECLARE_EVENT_TABLE()
 };
 class GDLWXStream;
@@ -2047,18 +2130,20 @@ public:
 
  void InitDrawSize(wxSize s) {drawSize = s;}
  wxSize GetDrawSize() {return drawSize;}
- 
+ wxPoint WhereIsMouse(wxMouseEvent &e);
+ wxPoint WhereIsMouse(wxKeyEvent &e);
  void ResizeDrawArea(const wxSize s);
  void DeleteUsingWindowNumber();
+ void SetUndecomposed();
  void SetStream(GDLWXStream* s);
 
+ void OnErase(wxEraseEvent& event);
  void OnPaint(wxPaintEvent& event);
  virtual void OnPlotWindowSize(wxSizeEvent &event);
  
 // virtual bool isPlot() const {return false;}
 
 
- void RepaintGraphics(bool doClear = false);
  virtual void RaisePanel()  {}
  virtual void LowerPanel()  {}
  virtual void IconicPanel()    {}
@@ -2100,11 +2185,29 @@ public:
   return true;
  }
 
-// virtual gdlWidgetDraw* GetGDLWidgetDraw() {
-//  return NULL;
-// }
  };
-//variant used in widget_draw, with specifics for widgets.
+// 
+////Stem for generalization of Drag'n'Drop, a WIDGET_DRAW can receive drop events from something else than a tree widget (but will probably misunderstand)
+//class DnDText : public wxTextDropTarget {
+//public:
+//
+//  DnDText() {
+//  }
+//
+//  virtual bool OnDropText(wxCoord x, wxCoord y, const wxString& text);
+//
+//};
+//
+//class DnDFile : public wxFileDropTarget
+//{
+//public:
+//    DnDFile() {  }
+//    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames) {
+//      std::cerr<<filenames[0]<<std::endl;
+//      return true;
+//    }
+//
+//};
 class gdlwxDrawPanel : public gdlwxGraphicsPanel
 {
  GDLWidgetDraw* myWidgetDraw;
@@ -2132,6 +2235,7 @@ public:
  void OnMouseUp(wxMouseEvent& event);
  void OnMouseWheel(wxMouseEvent& event);
  void OnKey(wxKeyEvent& event);
+ void OnFakeDropFileEvent(wxDropFilesEvent& event);
  //  void OnSize(wxSizeEvent &event);
 // void OnPlotWindowSize(wxSizeEvent &event);
 
