@@ -27,26 +27,25 @@
 
 using namespace std;
 
-void GDLXStream::Init() {
+void GDLXStream::FindTerminalWindow() {
   // plstream::init() calls exit() if it cannot establish a connection with X-server
-  bool okToRevertToTerminal=false;
   int revert_to;
-  {
-    Display* display = XOpenDisplay(NULL);
+  Display* display = XOpenDisplay(NULL);
+  if (display == NULL) {
+    display = XOpenDisplay(":0");//IDL also opens :0 when DISPLAY is not set.
     if (display == NULL) {
-      display = XOpenDisplay(":0");//IDL also opens :0 when DISPLAY is not set.
-      if (display == NULL) {
-      valid = false;
-      ThrowGDLException("Cannot connect to X server");
-      }
+    valid = false;
+    ThrowGDLException("Cannot connect to X server");
     }
-    XGetInputFocus(display, &term_window, &revert_to);
-    //avoid doing impossible things if term_window is *not* a bona fide window.
-    if (term_window == PointerRoot || term_window == None) term_window=0;
-    XCloseDisplay(display);
   }
+  XGetInputFocus(display, &term_window, &revert_to);
+  //avoid doing impossible things if term_window is *not* a bona fide window.
+  if (term_window == PointerRoot || term_window == None) term_window=0;
+  XCloseDisplay(display);
+}
 
-  this->plstream::init();
+void GDLXStream::PostInit() {
+  bool okToRevertToTerminal=false;
 
   XwDev *dev = (XwDev *) pls->dev;
   XwDisplay *xwd = (XwDisplay *) dev->xwd;
@@ -63,14 +62,6 @@ void GDLXStream::Init() {
   }
   if ( okToRevertToTerminal ) XSetInputFocus(xwd->display,term_window,RevertToParent,CurrentTime);
   else UnsetFocus(); //desperate method, since it prevents iconifying etc...
-//  XFlush(xwd->display);
-  GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
-  //current cursor:
-  CursorStandard(actDevice->getCursorId());
-  //current graphics function
-  SetGraphicsFunction(actDevice->GetGraphicsFunction());
-  //BackingStore 
-  SetBackingStore(actDevice->getBackingStore());
 }
 
 void GDLXStream::EventHandler() {
@@ -223,14 +214,13 @@ BaseGDL* GDLXStream::GetFontnames(DString pattern) {
   int actual_count_return;
   char **list= XListFonts(xwd->display, pattern.c_str(), 30000, &actual_count_return);
   if (!actual_count_return) return NULL;
-  DStringGDL* myList=new DStringGDL(dimension(actual_count_return));
+  DStringGDL* myList=new DStringGDL(dimension(actual_count_return),BaseGDL::NOZERO);
   for (int i=0; i< actual_count_return; ++i) (*myList)[i].assign(list[i]);
   XFreeFontNames(list);
   return myList;
 }
 DLong GDLXStream::GetFontnum(DString pattern){
-  // never occurs ...  if (pattern.length()<0) return -1;
-  if (this->GetFontnames(pattern) == NULL) return -1;
+  if (this->GetFontnames(pattern) == NULL) return 0;
   if (pattern.length()==0) return 0;
   return this->GetFontnames(pattern)->N_Elements();
 }
@@ -334,7 +324,7 @@ void GDLXStream::DeIconic() {
   XMapWindow(dev->xwd->display, dev->window);
 }
 
-void GDLXStream::UnMapWindow() {
+void GDLXStream::UnMapWindowAndSetPixmapProperty() {
   //Used for /PIXMAP windows: 1) insure write_to_pixmap and not write_to_window, and 2) hide the window.
   XwDev *dev = (XwDev *) pls->dev;
   dev->write_to_pixmap=1;
@@ -731,14 +721,6 @@ bool GDLXStream::PaintImage(unsigned char *idata, PLINT nx, PLINT ny, DLong *pos
   XColor curcolor;
   curcolor = xwd->fgcolor; //default
   PLINT iclr1, ired, igrn, iblu;
-  // parallelize does not work when using XGet[Put]Pixel in the loop below, otherwise would be OK!
-  // please allow parallelization only after removing this problem ;^)
-  //#ifdef _OPENMP
-  //  SizeT nOp = kxLimit * kyLimit;
-  //#endif
-  //  #pragma omp parallel if (nOp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nOp)) private(ired,igrn,iblu,kx,ky,iclr1,curcolor)
-  //  {
-  //  #pragma omp for
   for (ix = 0; ix < xmax; ++ix) {
     for (iy = 0; iy < ymax; ++iy) {
 
@@ -797,7 +779,6 @@ bool GDLXStream::PaintImage(unsigned char *idata, PLINT nx, PLINT ny, DLong *pos
       XPutPixel(ximg, ix, ymax-1-iy, curcolor.pixel); //ximg IS reversed allways.
     }
   }
-  //  } //end parallelize 
   if (dev->write_to_pixmap == 1)
     XPutImage(xwd->display, dev->pixmap, dev->gc, ximg, 0, 0,
     xoff, dev->height-yoff-ymax, xmax, ymax);
@@ -840,8 +821,7 @@ void GDLXStream::Color( ULong color, DLong decomposed ) {
   XSetForeground( xwd->display, dev->gc, curcolor);
 }
 
-//Read X11 bitmapdata -- normally on 4BPP=Allplanes, return 3BPP ignoring Alpha plane.
-DByteGDL* GDLXStream::GetBitmapData() {
+DByteGDL* GDLXStream::GetBitmapData(int xoff, int yoff, int nx, int ny) {
 //  plstream::cmd( PLESC_FLUSH, NULL );
   GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
   
@@ -852,15 +832,14 @@ DByteGDL* GDLXStream::GetBitmapData() {
 
   /* query the window's attributes. */
   Status rc = XGetWindowAttributes(xwd->display, dev->window, &win_attr);
-  unsigned int nx = win_attr.width;
-  unsigned int ny = win_attr.height;
+  unsigned int screen_ny = win_attr.height;
 
     int (*oldErrorHandler)(Display*, XErrorEvent*);
     oldErrorHandler = XSetErrorHandler(GetImageErrorHandler);
     if (dev->write_to_pixmap==1) {
-      ximg = XGetImage(xwd->display, dev->pixmap, 0, 0, nx, ny, AllPlanes, ZPixmap);
+      ximg = XGetImage(xwd->display, dev->pixmap, xoff,  screen_ny-ny-yoff, nx, ny, AllPlanes, ZPixmap);
     } else {
-      ximg = XGetImage( xwd->display, dev->window, 0, 0, nx, ny, AllPlanes, ZPixmap);
+      ximg = XGetImage( xwd->display, dev->window, xoff, screen_ny-ny-yoff, nx, ny, AllPlanes, ZPixmap);
     }
     XSetErrorHandler(oldErrorHandler);
     
@@ -876,15 +855,15 @@ DByteGDL* GDLXStream::GetBitmapData() {
     //PADDING is 4BPP -- we take 3BPP and revert Y to respect IDL default
     SizeT kpad = 0;
     for ( SizeT iy =0; iy < ny ; ++iy ) {
-      for ( SizeT ix = 0; ix < nx; ++ix ) {
-        (*bitmap)[3 * ((ny-1-iy) * nx + ix) + 2] = ximg->data[kpad++];
-        (*bitmap)[3 * ((ny-1-iy) * nx + ix) + 1] = ximg->data[kpad++];
-        (*bitmap)[3 * ((ny-1-iy) * nx + ix) + 0] = ximg->data[kpad++];
+      auto j0=3 * (ny-1-iy) * nx ;
+      for ( SizeT ix = 0; ix < 3*nx; ix+=3 ) {
+        (*bitmap)[ j0 + ix + 2] = ximg->data[kpad++];
+        (*bitmap)[ j0 + ix + 1] = ximg->data[kpad++];
+        (*bitmap)[ j0 + ix + 0] = ximg->data[kpad++];
         kpad++; //pad to 4
       }
     }
     XDestroyImage(ximg);
     return bitmap;
 }
-
 #endif
