@@ -28,16 +28,63 @@ class GDLXStream: public GDLGStream
   Atom wm_delete_window;
   Window term_window;
 public:
-  GDLXStream( int nx, int ny)
-    : GDLGStream( nx, ny, "xwin",XOpenDisplay(NULL)==NULL?":0":NULL) //IDL also opens :0 when DISPLAY is not set.
+  GDLXStream( PLINT xp , PLINT yp, PLINT nx, PLINT ny, PLINT xoff, PLINT yoff, const std::string &title)
+    : GDLGStream( nx, ny, "xwin") 
     , term_window(0)
-  {
+{
+    spage( xp , yp, nx, ny, xoff, yoff); 
+
+    // window title
+    static char buf[ 256];
+    strncpy(buf, title.c_str(), 255);
+    buf[ 255] = 0;
+    setopt("plwindow", buf);
+
+    // no pause on win destruction
+    plstream::spause(false);
+
+    // extended fonts
+    plstream::fontld(1);
+
+    // we want color
+    plstream::scolor(1);
+
+    PLINT r[ctSize], g[ctSize], b[ctSize];
+    GDLCT* myCT = GraphicsDevice::GetGUIDevice()->GetCT();
+    myCT->Get(r, g, b);
+    SetColorMap0(r, g, b, ctSize); //set colormap 0 to 256 values
+
+
+    // Do not init colors --- we handle colors ourseves, very much faster!
+    // Do not use command buffer
+    // Please no threads, no gain especially in remote X11 
+    setopt("drvopt", "noinitcolors=1,nobuffered=1,usepth=0");
+
+    FindTerminalWindow(); //to pro
+    //all the options must be passed BEFORE INIT=plinit.
+    init(); //creates the X11 window.
+    PostInit(); //finish setting different things, including giving back focus to terminal
+    
+    // need to be called initially. permit to fix things
+    plstream::ssub(1, 1); // plstream below stays with ONLY ONE page
+    plstream::adv(0); //-->this one is the 1st and only pladv
+    // load font
+    plstream::font(1);
+    plstream::vpor(0, 1, 0, 1);
+    plstream::wind(0, 1, 0, 1);
+
+    ssub(1, 1);
+    SetPageDPMM();
+    DefaultCharSize();
+    adv(0); //this is for us (counters) //needs DefaultCharSize
+    clear();
   }
 
   ~GDLXStream()
   {}
-  
-  void Init();
+  void Init(){};
+  void FindTerminalWindow();
+  void PostInit();
   void EventHandler();
 
   /*------------------------------------------------------------------------*\
@@ -84,8 +131,8 @@ public:
   bool HasSafeDoubleBuffering();
   bool PaintImage(unsigned char *idata, PLINT nx, PLINT ny,  DLong *pos, DLong tru, DLong chan);
   virtual bool HasCrossHair() {return true;}
-  void UnMapWindow();
-  DByteGDL* GetBitmapData();
+  void UnMapWindowAndSetPixmapProperty();
+  virtual DByteGDL* GetBitmapData(int xoff, int yoff, int nx, int ny) final;
   void Color( ULong color, DLong decomposed);
 //  void Update(){plstream::cmd(PLESC_EXPOSE, NULL);}
   
