@@ -26,14 +26,18 @@
 
 #include "datatypes.hpp"
 #include "envt.hpp"
+#include <vector>
+#include <cassert>
+#include <gsl/gsl_vector.h>
+#include <gsl/gsl_multifit_nlinear.h>
+#include <functional>
+
 
 namespace lib {
 
   //  BaseGDL* invert_fun( EnvT* e);
   BaseGDL* fft_fun( EnvT* e);
   BaseGDL* random_fun( EnvT* e);
-  BaseGDL* histogram_fun( EnvT* e);
-//  BaseGDL* interpolate_fun( EnvT* e);
 
   void la_trired_pro( EnvT* e);
 
@@ -64,6 +68,8 @@ namespace lib {
   BaseGDL* wtn(EnvT* e);
   BaseGDL* zeropoly(EnvT* e);
   BaseGDL* spher_harm(EnvT* e);
+  BaseGDL* gaussfit(EnvT* e);
+  BaseGDL* random_fun_gsl(EnvT* e);
 
   template< typename T1, typename T2>
   int cp2data2_template( BaseGDL* p0, T2* data, SizeT nEl, 
@@ -71,9 +77,8 @@ namespace lib {
   template< typename T>
   int cp2data_template( BaseGDL* p0, T* data, SizeT nEl, 
 			SizeT offset, SizeT stride_in, SizeT stride_out);
+    
 
-  BaseGDL* random_fun_gsl(EnvT* e);
-//  BaseGDL* interpol_fun(EnvT* e);
 } // namespace
 
 #endif
