@@ -53,12 +53,12 @@ void LibInit_cl()
   new DLibPro(lib::journal,string("JOURNAL"),1);
 
   const string timestampKey[]={"DAY","HOUR","MINUTE","MONTH","OFFSET","SECOND","UTC","YEAR","ZERO",KLISTEND};
-  new DLibFunRetNew(lib::timestamp,string("TIMESTAMP"),2,timestampKey);
+  new DLibFunRetNew(lib::timestamp,string("TIMESTAMP"),0,timestampKey);
 
   const string timestamptovaluesKey[]={"DAY","HOUR","MINUTE","MONTH","OFFSET","SECOND","YEAR",KLISTEND};
   new DLibPro (lib::timestamptovalues,string("TIMESTAMPTOVALUES"),1,timestamptovaluesKey);
 
-  const string systimeKey[]={"JULIAN","SECONDS","UTC",KLISTEND};
+  const string systimeKey[]={"ELAPSED","JULIAN","SECONDS","UTC",KLISTEND};
   new DLibFunRetNew(lib::systime,string("SYSTIME"),2,systimeKey);
 
   const string legendreKey[]={"DOUBLE",KLISTEND};
@@ -217,7 +217,7 @@ void LibInit_cl()
   const string magick_quantizeKey[]={"TRUECOLOR","YUV","GRAYSCALE","DITHER",KLISTEND};
   new DLibPro(lib::magick_quantize,string("MAGICK_QUANTIZE"),2,magick_quantizeKey);
  
-  new DLibPro(lib::magick_writeIndexes, string("MAGICK_WRITEINDEXES"),2);
+//  new DLibPro(lib::magick_writeIndexes, string("MAGICK_WRITEINDEXES"),2);
   new DLibPro(lib::magick_writeColorTable, string("MAGICK_WRITECOLORTABLE"),4);
 
 
