@@ -27,6 +27,7 @@
 #include "basic_pro_jmg.hpp"
 
 #include "gsl_fun.hpp"
+#include "histogram.hpp"
 #include "interpolate.hpp"
 #include "interpol.hpp"
 #include "image.hpp"
@@ -51,6 +52,10 @@
 #include "matrix_invert.hpp"
 #include "triangulation.hpp"
 
+#ifdef HAVE_QHULL
+#include "qhull.hpp"
+#endif
+
 using namespace std;
 
 void LibInit_jmg()
@@ -106,9 +111,9 @@ void LibInit_jmg()
   // if FFTw not available, FFT in the GSL used (slower)
   const string fftKey[]={"DOUBLE","INVERSE","OVERWRITE","DIMENSION","CENTER",KLISTEND};
 #if defined(USE_FFTW)
-  new DLibFun(lib::fftw_fun,string("FFT"),2,fftKey);
+  new DLibFun(lib::fftw_fun,string("FFT"),2,fftKey,NULL,0,true);  //UsesThreadPOOL 
 #else
-  new DLibFun(lib::fft_fun,string("FFT"),2,fftKey);
+  new DLibFun(lib::fft_fun,string("FFT"),2,fftKey); 
 #endif
 
   const string randomKey[]={"DOUBLE","GAMMA","LONG","NORMAL",
@@ -126,10 +131,10 @@ void LibInit_jmg()
   new DLibFunRetNew(lib::histogram_fun,string("HISTOGRAM"),1,histogramKey,histogramWarnKey);
   
   const string interpolKey[]={ "LSQUADRATIC","NAN", "QUADRATIC", "SPLINE" ,KLISTEND};
-  new DLibFunRetNew(lib::interpol_fun,string("INTERPOL"),3,interpolKey);
+  new DLibFunRetNew(lib::interpol_fun,string("GDL_INTERPOL"),3,interpolKey); //internal function, we use interpol.pro to sort x and y in the 3 parameter case.
   
   const string interpolateKey[]={"CUBIC","DOUBLE","GRID","MISSING","NEAREST_NEIGHBOUR",KLISTEND};
-  new DLibFunRetNew(lib::interpolate_fun,string("INTERPOLATE"),4,interpolateKey);
+  new DLibFunRetNewTP(lib::interpolate_fun,string("INTERPOLATE"),4,interpolateKey);  //UsesThreadPOOL 
 
   const string la_triredKey[]={"DOUBLE","UPPER",KLISTEND};
   new DLibPro(lib::la_trired_pro,string("LA_TRIRED"),3,la_triredKey);
@@ -140,12 +145,10 @@ void LibInit_jmg()
 
 #if defined(USE_LIBPROJ)
   const string map_proj_forwardKey[]={"MAP_STRUCTURE","RADIANS","POLYGONS","POLYLINES","CONNECTIVITY","FILL",KLISTEND};  //WARNING FIXED ORDER for GetMapAsMapStructureKeyword()
-  new DLibFunRetNew(lib::map_proj_forward_fun,
-	      string("MAP_PROJ_FORWARD"),2,map_proj_forwardKey,NULL);
+  new DLibFunRetNewTP(lib::map_proj_forward_fun,string("MAP_PROJ_FORWARD"),2,map_proj_forwardKey,NULL);  //UsesThreadPOOL 
 
   const string map_proj_inverseKey[]={"MAP_STRUCTURE","RADIANS",KLISTEND}; //WARNING FIXED ORDER for GetMapAsMapStructureKeyword()
-  new DLibFunRetNew(lib::map_proj_inverse_fun,
-	      string("MAP_PROJ_INVERSE"),2,map_proj_inverseKey);
+  new DLibFunRetNewTP(lib::map_proj_inverse_fun,string("MAP_PROJ_INVERSE"),2,map_proj_inverseKey);  //UsesThreadPOOL 
 //dummy functions for compatibility support of GCTP projections 
   new DLibPro(lib::map_proj_gctp_forinit,string("MAP_PROJ_GCTP_FORINIT"),4);
   new DLibPro(lib::map_proj_gctp_revinit,string("MAP_PROJ_GCTP_REVINIT"),4);
@@ -160,12 +163,12 @@ void LibInit_jmg()
 
 
   const string finiteKey[]={"INFINITY","NAN","SIGN",KLISTEND};
-  new DLibFunRetNew(lib::finite_fun,string("FINITE"),1,finiteKey);
+  new DLibFunRetNewTP(lib::finite_fun,string("FINITE"),1,finiteKey);  //UsesThreadPOOL 
 
   const string radonKey[]={"BACKPROJECT","DOUBLE","DRHO","DX","DY",
 			   "GRAY","LINEAR","NRHO","NTHETA","NX","NY",
 			   "RHO","RMIN","THETA","XMIN","YMIN",KLISTEND};
-  new DLibFunRetNew(lib::radon_fun,string("RADON"),1,radonKey);
+  new DLibFunRetNewTP(lib::radon_fun,string("RADON"),1,radonKey);
   
 
   const string grid_inputKey[]={"SPHERE", "POLAR", "DEGREES", "DUPLICATES", "EPSILON", "EXCLUDE", KLISTEND};
@@ -190,14 +193,14 @@ void LibInit_jmg()
   new DLibFunRetNew(lib::trigrid_fun,string("TRIGRID"),6,trigridKey,trigridWarnKey);
 
   const string poly_2dKey[]={"CUBIC","MISSING",KLISTEND};
-  new DLibFunRetNew(lib::poly_2d_fun,string("POLY_2D"),6,poly_2dKey);
+  new DLibFunRetNewTP(lib::poly_2d_fun,string("POLY_2D"),6,poly_2dKey);  //UsesThreadPOOL 
 
   const string make_arrayKey[]={"DIMENSION", "INCREMENT", "INDEX", "NOZERO",
                                  "SIZE", "START", "TYPE", "VALUE", "BOOLEAN",
                                  "BYTE", "COMPLEX", "DCOMPLEX", "DOUBLE",
                                  "FLOAT", "INTEGER", "L64", "LONG", "OBJ",
                                  "PTR", "STRING", "UINT", "UL64", "ULONG", KLISTEND};
-  new DLibFunRetNew(lib::make_array,string("MAKE_ARRAY"),MAXRANK,make_arrayKey);
+  new DLibFunRetNewTP(lib::make_array,string("MAKE_ARRAY"),MAXRANK,make_arrayKey);  //UsesThreadPOOL //not yet
 
   const string reformKey[]={"OVERWRITE",KLISTEND};
   new DLibFun(lib::reform,string("REFORM"),MAXRANK+1,reformKey);
@@ -304,7 +307,9 @@ void LibInit_jmg()
  
 #endif
 
-  const string tvKey[]={"TRUE","NORMAL","CHANNEL","XSIZE","YSIZE","ORDER","DEVICE","DATA","T3D","Z","CENTIMETERS","INCHES",KLISTEND};
+  const string tvKey[]={"TRUE","NORMAL","CHANNEL","XSIZE","YSIZE","ORDER","DEVICE","DATA","T3D","Z","CENTIMETERS","INCHES",
+  "NAN",  //byte conversion in tv_image apparently handle NaNs quite gracefully so the /NAN (hidden) option of TV will be ignored (TBC)
+  KLISTEND};
   const string tvWarnKey[]={"WORDS",KLISTEND};
   new DLibPro(lib::tv_image,string("TV"),4,tvKey,tvWarnKey);
 
