@@ -48,7 +48,6 @@
     if (notInitialized ) { \
       notInitialized = false; \
         Magick::InitializeMagick(NULL);  \
-        if ( QuantumDepth < 16) fprintf(stderr, "%% INFO: the %s library support only 8 bit images.\n", MagickPackageName);\
               }
 
 namespace lib {
@@ -78,7 +77,9 @@ namespace lib {
 
   Image *magick_image(EnvT *e, unsigned int mid) {
     if (gValid[mid] == 0) e->Throw("invalid ID.");
-    return gImage[mid];
+    Image* im=gImage[mid];
+//    std::cerr<<im->depth()<<std::endl;
+    return im;
   }
 
   unsigned int magick_image(EnvT* e, Image *imImage) {
@@ -206,7 +207,7 @@ namespace lib {
       // Despite Type is NOT useful without a a->read(), it is OK for Palette !
       // This should be reliable (OK with ImageMagick AND GraphicsMagick)
       DInt has_palette = 0;
-      if ( (a->type() == PaletteType) || (a->type() == PaletteMatteType) ) has_palette = 1;
+      if ( a->classType() == PseudoClass) has_palette = 1;
 
       DString type;
       type = a->magick() == "PNM" ? "PPM" :
@@ -382,8 +383,13 @@ namespace lib {
 #endif
       {
         DByteGDL *bImage = new DByteGDL(dim, BaseGDL::ZERO);
+#ifndef HAS_IMAGEMAGICK
         PixelPacket *pixel_cache = image->getPixels(0,0,columns,rows); //magick command, without it writePixels do NOTHING!
         image->writePixels(IndexQuantum,(unsigned char*)(bImage->DataAddr()));
+#else
+        Quantum *pixel_cache = image->getPixels(0,0,columns,rows); //magick command, without it writePixels do NOTHING!
+        image->writePixels(IndexQuantum,(unsigned char*)(bImage->DataAddr()));
+#endif
         return bImage;
       }
 #ifdef USE_MAGICK6
@@ -393,8 +399,13 @@ namespace lib {
 #endif
       {
         DUIntGDL *bImage = new DUIntGDL(dim, BaseGDL::NOZERO);
-        PixelPacket *pixel_cache = image->getPixels(0,0,columns,rows); //magick command, without it writePixels do NOTHING!
-        image->writePixels(IndexQuantum,(unsigned char*)(bImage->DataAddr())); //probably not good, impossible to test yet.
+#ifndef HAS_IMAGEMAGICK
+        PixelPacket *pixel_cache = image->getPixels(0, 0, columns, rows); //magick command, without it writePixels do NOTHING!
+        image->writePixels(IndexQuantum, (unsigned char*) (bImage->DataAddr()));
+#else
+        Quantum *pixel_cache = image->getPixels(0, 0, columns, rows); //magick command, without it writePixels do NOTHING!
+        image->writePixels(IndexQuantum, (unsigned char*) (bImage->DataAddr()));
+#endif
         return bImage;
       }
     } catch (Exception &error_) {
@@ -608,14 +619,18 @@ namespace lib {
             if (image->matte()) map = map + "A";
           }
         }
-        if (image->depth() == 8) {
+        if (image->depth() == 8 || GDLimage->Type()==GDL_BYTE || !(IntType(GDLimage->Type()))) { //only INTeger values > BYTE can be saved as 16 bits see WRITE_PNG help
           DByteGDL * bImage = static_cast<DByteGDL*> (GDLimage->Convert2(GDL_BYTE, BaseGDL::COPY));
           Guard<DByteGDL> bImageGuard(bImage);
+        // Ensure that there are no other references to this image.
+          image->modifyImage();
           image->read(columns, rows, map, CharPixel, &(*bImage)[0]);
-        } else if (image->depth() == 16) {
-          DUIntGDL * iImage = static_cast<DUIntGDL*> (GDLimage->Convert2(GDL_UINT, BaseGDL::COPY));
-          Guard<DUIntGDL> iImageGuard(iImage);
-          image->read(columns, rows, map, ShortPixel, &(*iImage)[0]);
+        } else if (image->depth() == 16) { //the only one able to support writng INTeger values > BYTES
+            DUIntGDL * iImage = static_cast<DUIntGDL*> (GDLimage->Convert2(GDL_UINT, BaseGDL::COPY));
+            Guard<DUIntGDL> iImageGuard(iImage);
+        // Ensure that there are no other references to this image.
+            image->modifyImage();
+            image->read(columns, rows, map, ShortPixel, &(*iImage)[0]);
         } else {
           e->Throw("Unsupported bit depth");
         }
@@ -633,7 +648,7 @@ namespace lib {
         if (image->colorMapSize() < 1) e->Throw("GDL internal: destination image has no colormap!");
         image->size(Geometry(columns, rows));
         image->setPixels(0,0,columns, rows);
-        image->readPixels(IndexQuantum,(unsigned char*)bImage->DataAddr());
+        image->read(columns, rows, "I", CharPixel, &(*bImage)[0]);
         image->syncPixels();
       }
       image->flip();
@@ -913,7 +928,7 @@ namespace lib {
 
       static int TRUECOLORIx = e->KeywordIx("TRUECOLOR");
       static int DITHERIx = e->KeywordIx("DITHER");
-      bool dither=true;
+      bool dither=false;
       if (e->GetKW(DITHERIx) !=NULL) dither=e->KeywordSet(DITHERIx);
       static int YUVIx = e->KeywordIx("YUV");
       static int GRAYSCALEIx = e->KeywordIx("GRAYSCALE");
@@ -924,10 +939,11 @@ namespace lib {
           image->quantizeColorSpace(GRAYColorspace);
         else
           image->quantizeColorSpace(RGBColorspace);
-
+        
       image->quantizeColors(ncol);
       image->quantizeDither(dither);
       image->quantize();
+//      image->quantizeTreeDepth(cube_depth);
       //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
@@ -961,44 +977,41 @@ namespace lib {
     image->display();
 
   }
-
-  void magick_writeIndexes(EnvT* e) {
-    START_MAGICK;
-    try {
-      DUInt mid;
-      e->AssureScalarPar<DUIntGDL>(0, mid);
-      BaseGDL* GDLimage = e->GetParDefined(1);
-      DByteGDL * bImage = static_cast<DByteGDL*> (GDLimage->Convert2(GDL_BYTE, BaseGDL::COPY));
-
-      Image* image = magick_image(e, mid);
-
-//      const PixelPacket* pixels;
-      IndexPacket* index;
-
-      unsigned int columns, rows;
-      columns = image->columns();
-      rows = image->rows();
-
-//      pixels = image->setPixels(0, 0, columns, rows);
-      index = image->getIndexes();
-
-      SizeT nEl = columns*rows;
-      // #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
-      {
-        // #pragma omp for
-        for (SizeT cx = 0; cx < nEl; ++cx) {
-          index[cx] = static_cast<unsigned int> ((*bImage)[cx]);
-          /*	    *index=(unsigned int)(*bImage)[cx];
-                  index++;*/
-        }
-      }
-      image->syncPixels();
-
-      //magick_replace(e, mid, image);
-    } catch (Exception &error_) {
-      e->Throw(error_.what());
-    }
-  }
+// //NOT USED -- AND WHAT SOES IT DO?
+//  void magick_writeIndexes(EnvT* e) {
+//    START_MAGICK;
+//    try {
+//      DUInt mid;
+//      e->AssureScalarPar<DUIntGDL>(0, mid);
+//      BaseGDL* GDLimage = e->GetParDefined(1);
+//      DByteGDL * bImage = static_cast<DByteGDL*> (GDLimage->Convert2(GDL_BYTE, BaseGDL::COPY));
+//
+//      Image* image = magick_image(e, mid);
+//#ifndef HAS_IMAGEMAGICK
+////      const MagicCore::PixelPacket* pixels;
+//      IndexPacket* index;
+//
+//      unsigned int columns, rows;
+//      columns = image->columns();
+//      rows = image->rows();
+//
+////      pixels = image->setPixels(0, 0, columns, rows);
+//      index = image->getIndexes();
+//
+//      SizeT nEl = columns*rows;
+//      {
+//        for (SizeT cx = 0; cx < nEl; ++cx) {
+//          index[cx] = static_cast<unsigned int> ((*bImage)[cx]);
+//        }
+//      }
+//#endif
+//      image->syncPixels();
+//
+//      //magick_replace(e, mid, image);
+//    } catch (Exception &error_) {
+//      e->Throw(error_.what());
+//    }
+//  }
 
   //MAGICK_WRITECOLORTABLE,mid[,r,g,b]
 
