@@ -16,7 +16,6 @@
  ***************************************************************************/
 
 /*
-
   using the Besel functions provided by GSL
 
   http://www.physics.ohio-state.edu/~ntg/780/gsl_examples/J0_test.cpp
@@ -65,7 +64,7 @@
   DType t0 = e->GetParDefined(0)->Type();				\
 
 //no "//" comments in macro !!!!
-//if (t0 == GDL_COMPLEX || t0 == GDL_COMPLEXDBL)		\
+//if (t0 == GDL_COMPLEX || t0 == GDL_COMPLEXDBL)		
 //  e->Throw("Complex not implemented (GSL limitation). ");
 
 #define AC_2P1()							\
@@ -93,8 +92,8 @@
 									\
   const double dzero = 0.0000000000000000000 ;				\
 									\
-  //    throw GDLException(e->CallingNode(), "Variable is undefined: "+e->GetParString(1)); \
-  									\
+  //    throw GDLException(e->CallingNode(), "Variable is undefined: "+e->GetParString(1)); 
+  									
   //  DType t1 = e->GetParDefined(1)->Type();				\
   //  if (t1 == GDL_COMPLEX || t1 == GDL_COMPLEXDBL)			\
   // e->Throw("Complex not implemented (GSL limitation). ");
@@ -539,6 +538,9 @@ namespace lib {
   // what does not work like IDL : warning messages when Inf/Nan or Zero/Negative X steps, X and Y not same size
 #define SPL_INIT_BIG double(1.0E30) 
   BaseGDL* spl_init_fun(EnvT* e) {
+    
+    e->NParam(2);
+
     static DInt doubleKWIx = e->KeywordIx("DOUBLE");
     bool isDouble = (e->GetParDefined(0)->Type() == GDL_DOUBLE || e->GetParDefined(1)->Type() == GDL_DOUBLE);
     if (e->KeywordSet(doubleKWIx)) isDouble = true;
@@ -579,7 +581,7 @@ namespace lib {
       firstderiv = yp1Ix;
       Yderiv0 = e->GetKW(firstderiv);
     }
-    DDoubleGDL* YP0;
+    DDoubleGDL* YP0=NULL;
     bool Yderiv0ok=false;
     if (Yderiv0 != NULL ) {
       YP0 = e->GetKWAs<DDoubleGDL>(firstderiv);
@@ -702,6 +704,7 @@ namespace lib {
     for (count = nElpXpos - 2; count != -1; --count) {
       (*res)[count] = (*res)[count]*(*res)[count + 1] + U[count];
     }
+    free(U); // see issue 1428
   givebackres:
     GM_CV0();
 
@@ -709,6 +712,9 @@ namespace lib {
 
   BaseGDL* spl_interp_fun( EnvT* e)
   {
+    
+    e->NParam(4);
+
     static DInt doubleKWIx = e->KeywordIx("DOUBLE");
     bool isDouble = (e->GetParDefined(0)->Type() == GDL_DOUBLE || e->GetParDefined(1)->Type() == GDL_DOUBLE || 
         e->GetParDefined(2)->Type() == GDL_DOUBLE || e->GetParDefined(3)->Type() == GDL_DOUBLE);
@@ -860,38 +866,38 @@ namespace lib {
       e->Throw( "Array must have 2 dimensions: "+ e->GetParString(0));
 
     switch (p0->Type()) {
-    case GDL_BYTE:{ long int a;
+    case GDL_BYTE:{ long int a=0;
       return Sobel_Template<DIntGDL>(static_cast<DByteGDL*>(p0),a);
     }
-    case GDL_INT: {long int a;
+    case GDL_INT: {long int a=0;
 	return Sobel_Template<DIntGDL>(static_cast<DIntGDL*> (p0),a);
     }
-    case GDL_UINT:{long int a;
+    case GDL_UINT:{long int a=0;
 	return Sobel_Template<DUIntGDL>(static_cast<DUIntGDL*> (p0),a);
     }
-    case GDL_LONG:{long int a;
+    case GDL_LONG:{long int a=0;
 	return Sobel_Template<DLongGDL>(static_cast<DLongGDL*> (p0),a);
     }
-    case GDL_ULONG:{long int a;
+    case GDL_ULONG:{long int a=0;
 	return Sobel_Template<DULongGDL>(static_cast<DULongGDL*> (p0),a);
     }
-    case GDL_LONG64:{DLong64 a;
+    case GDL_LONG64:{DLong64 a=0;
 	return Sobel_Template<DLong64GDL>(static_cast<DLong64GDL*> (p0),a);
     }
-    case GDL_ULONG64:{DLong64 a;
+    case GDL_ULONG64:{DLong64 a=0;
 	return Sobel_Template<DULong64GDL>(static_cast<DULong64GDL*> (p0),a);
     }
-    case GDL_FLOAT:{long double a;
+    case GDL_FLOAT:{long double a=0;
 	return Sobel_Template_floatingpointTypes<DFloatGDL>(static_cast<DFloatGDL*> (p0),a);
     }
-    case GDL_DOUBLE: {long double a;
+    case GDL_DOUBLE: {long double a=0;
 	return Sobel_Template_floatingpointTypes<DDoubleGDL>(static_cast<DDoubleGDL*> (p0),a);
     }
-    case GDL_COMPLEX: { long double a;
+    case GDL_COMPLEX: { long double a=0;
 	DDoubleGDL* p0 = e->GetParAs<DDoubleGDL>(0);
 	return Sobel_Template_floatingpointTypes<DComplexGDL>(static_cast<DDoubleGDL*> (p0),a);
     }
-    case GDL_COMPLEXDBL:{long double a;
+    case GDL_COMPLEXDBL:{long double a=0;
 	DDoubleGDL* p0 = e->GetParAs<DDoubleGDL>(0);
 	return Sobel_Template_floatingpointTypes<DComplexDblGDL>(static_cast<DDoubleGDL*> (p0),a);
     }
@@ -1008,38 +1014,38 @@ namespace lib {
 
     switch (p0->Type()) {
     case GDL_BYTE:{
-      long int a;
+      long int a=0;
       return Prewitt_Template<DIntGDL>(static_cast<DByteGDL*>(p0),a);
     }
-    case GDL_INT: {long int a;
+    case GDL_INT: {long int a=0;
 	return Prewitt_Template<DIntGDL>(static_cast<DIntGDL*> (p0),a);
     }
-    case GDL_UINT:{long int a;
+    case GDL_UINT:{long int a=0;
 	return Prewitt_Template<DUIntGDL>(static_cast<DUIntGDL*> (p0),a);
     }
-    case GDL_LONG:{long int a;
+    case GDL_LONG:{long int a=0;
 	return Prewitt_Template<DLongGDL>(static_cast<DLongGDL*> (p0),a);
     }
-    case GDL_ULONG:{long int a;
+    case GDL_ULONG:{long int a=0;
 	return Prewitt_Template<DULongGDL>(static_cast<DULongGDL*> (p0),a);
     }
-    case GDL_LONG64:{long int a;
+    case GDL_LONG64:{long int a=0;
 	return Prewitt_Template<DLong64GDL>(static_cast<DLong64GDL*> (p0),a);
     }
-    case GDL_ULONG64:{long int a;
+    case GDL_ULONG64:{long int a=0;
 	return Prewitt_Template<DULong64GDL>(static_cast<DULong64GDL*> (p0),a);
     }
-    case GDL_FLOAT:{long int a;
+    case GDL_FLOAT:{long int a=0;
 	return Prewitt_Template<DFloatGDL>(static_cast<DFloatGDL*> (p0),a);
     }
-    case GDL_DOUBLE: {long int a;
+    case GDL_DOUBLE: {long int a=0;
 	return Prewitt_Template<DDoubleGDL>(static_cast<DDoubleGDL*> (p0),a);
     }
-    case GDL_COMPLEX: { long int a;
+    case GDL_COMPLEX: { long int a=0;
 	DDoubleGDL* p0 = e->GetParAs<DDoubleGDL>(0);
 	return Prewitt_Template<DComplexGDL>(static_cast<DDoubleGDL*> (p0),a);
     }
-    case GDL_COMPLEXDBL:{long int a;
+    case GDL_COMPLEXDBL:{long int a=0;
 	DDoubleGDL* p0 = e->GetParAs<DDoubleGDL>(0);
 	return Prewitt_Template<DComplexDblGDL>(static_cast<DDoubleGDL*> (p0),a);
     }
@@ -1051,7 +1057,7 @@ namespace lib {
 
   BaseGDL* erode_fun( EnvT* e){
 
-    SizeT nParam = e->NParam(1);
+    SizeT nParam = e->NParam(2);
 
     DIntGDL* p0 = e->GetParAs<DIntGDL>(0);
     DIntGDL* p1 = e->GetParAs<DIntGDL>(1);
@@ -1213,7 +1219,7 @@ namespace lib {
 
   BaseGDL* dilate_fun( EnvT* e)
   {
-    SizeT nParam = e->NParam(1);
+    SizeT nParam = e->NParam(2);
 
     DIntGDL* p0 = e->GetParAs<DIntGDL>(0);
     DIntGDL* p1 = e->GetParAs<DIntGDL>(1);
@@ -1368,6 +1374,9 @@ namespace lib {
 
   BaseGDL* matrix_multiply( EnvT* e)
   {
+     
+    e->NParam(2);
+
     BaseGDL* a = e->GetParDefined(0);
     BaseGDL* b = e->GetParDefined(1);
 
