@@ -42,7 +42,7 @@
 
 
 
-#define GDL_DEBUG
+//#define GDL_DEBUG
 //#undef GDL_DEBUG
 
 namespace lib {
