@@ -29,7 +29,7 @@
 #include "gdlexception.hpp"
 
 // print out AST tree
-#define GDL_DEBUG
+//#define GDL_DEBUG
 //#undef GDL_DEBUG
 
 #ifdef GDL_DEBUG
