@@ -57,9 +57,11 @@ extern GDLFileListT  fileUnits;
 // for OpenMP
 const SizeT DefaultTPOOL_MIN_ELTS = 100000;
 const SizeT DefaultTPOOL_MAX_ELTS = 0;
-extern DLong CpuTPOOL_NTHREADS;
-extern DLong64 CpuTPOOL_MIN_ELTS;
-extern DLong64 CpuTPOOL_MAX_ELTS;
+
+//defined as extern in typedefs.hpp, which is 'seen' by all code
+//extern DLong CpuTPOOL_NTHREADS;
+//extern DLong64 CpuTPOOL_MIN_ELTS;
+//extern DLong64 CpuTPOOL_MAX_ELTS;
 
 //extern DeviceListT   deviceList;
 //extern Graphics*     actDevice;
@@ -67,24 +69,37 @@ extern DLong64 CpuTPOOL_MAX_ELTS;
 // signals if control-c was pressed
 extern volatile bool sigControlC;
 
+//this string contains the value of DATADIR
+extern std::string gdlDataDir;
+//this string contains the value of LIBDIR
+extern std::string gdlLibDir;
+
 extern volatile bool iAmANotebook;
 // tells if wxwidgets is to be used at all...
 extern volatile bool useWxWidgets;
 // tells if wxwidgets backend for graphics is to be used...
 extern volatile bool useWxWidgetsForGraphics;
 // do we force fonts to be the ugly IDL fonts?
-extern volatile bool forceWxWidgetsUglyFonts;
+extern volatile bool tryToMimicOriginalWidgets;
 //do we favor SIMD-accelerated random number generation?
 extern volatile bool useDSFMTAcceleration;
+//do we use our own copy of (better?) drivers?
+extern volatile bool useLocalDrivers;
 extern volatile bool usePlatformDeviceName;
+extern volatile bool useEigenForTransposeOps;
+extern volatile bool useSmartTpool;
 extern          int  debugMode;
 
 enum DebugCode {
   DEBUG_CLEAR=0,
-  DEBUG_STOP = 1,
-  DEBUG_PROCESS_STOP = 2,
+  DEBUG_OUT = 1,
+  DEBUG_RETURN = 2,
   DEBUG_STEP = 3,
-  DEBUG_STEPOVER= 4
+  DEBUG_STEPOVER = 4,
+  DEBUG_TRACE = 5,
+  DEBUG_STOP_SILENT = 6,
+  DEBUG_STOP = 7,
+  DEBUG_PROCESS_STOP = 8
 };
 
 template< class Container> void PurgeContainer( Container& s) 
@@ -113,7 +128,6 @@ bool IsRelaxed(); //tells if syntax is not strict (i.e. parenthesis for array in
 void SetStrict(bool value);
 
 bool BigEndian();
-
 int get_suggested_omp_num_threads();
 int currentNumberOfThreads();
 int currentThreadNumber();
