@@ -17,35 +17,56 @@
 
 #include "includefirst.hpp"
 #include "plotting.hpp"
+#include "dinterpreter.hpp"
+
+#define GDL_PI     double(3.1415926535897932384626433832795)
 
 namespace lib
 {
   using namespace std;
 
 // shared parameter
-  static PLFLT lightSourcePos[3]={1.0,1.0,0.0};
+  static PLFLT lightSourcePos[3]={0,0,1.0};
 
   class shade_surf_call: public plotting_routine_call
   {
     DDoubleGDL *zVal, *yVal, *xVal;
     Guard<BaseGDL> xval_guard, yval_guard, zval_guard, p0_guard;
     SizeT xEl, yEl, zEl;
-    DDouble xStart, xEnd, yStart, yEnd, zStart, zEnd, datamax, datamin;
+    DDouble xStart, xEnd, yStart, yEnd, zStart, zEnd, zValue, datamax, datamin;
+    DDouble minVal,maxVal;
+    bool hasMinVal, hasMaxVal;
+    bool doT3d;
     bool nodata;
     bool setZrange;
     bool xLog;
     bool yLog;
     bool zLog;
-    ORIENTATION3D axisExchangeCode;
-    DLongGDL *shades;
-    Guard<BaseGDL> shades_guard;
-    bool doShade;
+    T3DEXCHANGECODE axisExchangeCode;
+    PLFLT alt = 30.0;
+    PLFLT az = 30.0;
+    PLFLT ay = 0;
+    bool below=false;
+    DDouble* Current3DMatrix;
+    DDoubleGDL* gdlBox3d;
+    
  private:
     bool handle_args (EnvT* e)
     {
+      //T3D ?
+      static int t3dIx = e->KeywordIx("T3D");
+      doT3d = (e->BooleanKeywordSet(t3dIx) || T3Denabled());
+
+      zValue = std::numeric_limits<DDouble>::quiet_NaN(); //NAN = no zValue?
+      static int zvIx = e->KeywordIx("ZVALUE");
+      if (e->KeywordPresent(zvIx)){
+        e->AssureDoubleScalarKWIfPresent(zvIx, zValue);
+        zValue = min(zValue, ZVALUEMAX); //to avoid problems with plplot
+        zValue = max(zValue, 0.0);
+      }
+      
      // in all cases, we have to exit here
-      if ( nParam()==2 || nParam()>3 )
-	{
+      if ( nParam()==2 || nParam()>3 )	{
 	  e->Throw ( "Incorrect number of arguments." );
 	}
 
@@ -58,37 +79,20 @@ namespace lib
     static int xLogIx = e->KeywordIx( "XLOG" );
     static int yLogIx = e->KeywordIx( "YLOG" );
     static int zLogIx = e->KeywordIx( "ZLOG" );
-    static int xTickunitsIx = e->KeywordIx( "XTICKUNITS" );
-    static int yTickunitsIx = e->KeywordIx( "YTICKUNITS" );
-    static int zTickunitsIx = e->KeywordIx( "ZTICKUNITS" );
 
     if ( e->KeywordPresent( xTypeIx ) ) xLog = e->KeywordSet( xTypeIx ); else xLog = e->KeywordSet( xLogIx );
     if ( e->KeywordPresent( yTypeIx ) ) yLog = e->KeywordSet( yTypeIx ); else yLog = e->KeywordSet( yLogIx );
     if ( e->KeywordPresent( zTypeIx ) ) zLog = e->KeywordSet( zTypeIx ); else zLog = e->KeywordSet( zLogIx );
 
-    if ( xLog && e->KeywordSet( xTickunitsIx ) ) {
-      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
-      xLog = FALSE;
-    }
-    if ( yLog && e->KeywordSet( yTickunitsIx ) ) {
-      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
-      yLog = FALSE;
-    }
-    if ( zLog && e->KeywordSet( zTickunitsIx ) ) {
-      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
-      zLog = FALSE;
-    }
-
-    if ( nParam ( ) > 0 )
-    {
+    if ( nParam ( ) > 0 ) {
 	// By testing here using EquivalentRank() we avoid computing zval if there was a problem.
 	// AC 2018/04/24
 	// a sub-array like: a=RANDOMU(seed, 3,4,5) & (this procedure name), a[1,*,*]
 	// should be OK ...
 	    if ( (e->GetNumericArrayParDefined ( 0 ))->EquivalentRank ( )!=2 ) e->Throw ( "Array must have 2 dimensions: "+e->GetParString ( 0 ) );
     }
-      if ( nParam ( )==1 )
-      {
+
+    if ( nParam ( )==1) {
        BaseGDL* p0=e->GetNumericArrayParDefined ( 0 )->Transpose ( NULL );
         p0_guard.Init ( p0 ); // delete upon exit
 
@@ -104,41 +108,34 @@ namespace lib
         yVal=new DDoubleGDL ( dimension ( yEl ), BaseGDL::INDGEN );
         yval_guard.Init ( yVal ); // delete upon exit
         if (yLog) yVal->Inc();
-      }
-      else
-{
+      } else {
         BaseGDL* p0 = e->GetNumericArrayParDefined(0)->Transpose(NULL);
         p0_guard.Init(p0); // delete upon exit
 
         zVal = static_cast<DDoubleGDL*> (p0->Convert2(GDL_DOUBLE, BaseGDL::COPY));
         zval_guard.Init(zVal); // delete upon exit
 
-        xVal = e->GetParAs< DDoubleGDL>(1);
-        yVal = e->GetParAs< DDoubleGDL>(2);
-
+        xVal = e->GetWriteableParAs< DDoubleGDL>(1);
+        yVal = e->GetWriteableParAs< DDoubleGDL>(2);
+          //filter out incompatible ranks >2 or ==0
         if (xVal->Rank() > 2)
           e->Throw("X, Y, or Z array dimensions are incompatible.");
-
         if (yVal->Rank() > 2)
           e->Throw("X, Y, or Z array dimensions are incompatible.");
         if (xVal->Rank() == 0 || yVal->Rank() == 0)
           e->Throw("X, Y, or Z array dimensions are incompatible.");
-
+          //filter out incompatible 1D dimensions
         if (xVal->Rank() == 1) {
           xEl = xVal->Dim(0);
-
           if (xEl != zVal->Dim(1))
             e->Throw("X, Y, or Z array dimensions are incompatible.");
         }
-
         if (yVal->Rank() == 1) {
           yEl = yVal->Dim(0);
-
           if (yEl != zVal->Dim(0))
             e->Throw("X, Y, or Z array dimensions are incompatible.");
         }
-
-
+          //filter out incompatible 2D dimensions
         if (xVal->Rank() == 2) {
           //plplot is unable to handle such subtetlies, better to throw?
           e->Throw("Sorry, plplot cannot handle 2D X coordinates in its 3D plots.");
@@ -146,7 +143,6 @@ namespace lib
           if ((xVal->Dim(0) != zVal->Dim(1))&&(xVal->Dim(1) != zVal->Dim(0)))
             e->Throw("X, Y, or Z array dimensions are incompatible.");
         }
-
         if (yVal->Rank() == 2) {
         //plplot is unable to handle such subtetlies, better to throw?
           e->Throw("Sorry, plplot cannot handle 2D Y coordinates in its 3D plots.");
@@ -179,13 +175,21 @@ namespace lib
       DDouble xAxisStart, xAxisEnd, yAxisStart, yAxisEnd;
       bool setx=gdlGetDesiredAxisRange(e, XAXIS, xAxisStart, xAxisEnd);
       bool sety=gdlGetDesiredAxisRange(e, YAXIS, yAxisStart, yAxisEnd);
-      if (setx) {
+      if (setx && sety) {
         xStart=xAxisStart;
         xEnd=xAxisEnd;
-      }
-      if (sety) {
         yStart=yAxisStart;
         yEnd=yAxisEnd;
+      } else if (sety) {
+        yStart = yAxisStart;
+        yEnd = yAxisEnd;
+      } else if (setx) {
+        xStart = xAxisStart;
+        xEnd = xAxisEnd;
+        //must compute min-max for other axis!
+        {
+          gdlDoRangeExtrema(xVal, yVal, yStart, yEnd, xStart, xEnd, false);
+        }
       }
       // z range
       datamax=0.0;
@@ -195,78 +199,19 @@ namespace lib
       zEnd=datamax;
       setZrange = gdlGetDesiredAxisRange(e, ZAXIS, zStart, zEnd);
 
-      //SHADES: Doing the job will be for nothing since plplot does not give the functionality.
-      static int shadesIx=e->KeywordIx ( "SHADES" ); doShade=false;
-      if ( e->GetKW ( shadesIx )!=NULL )
-      {
-        shades=e->GetKWAs<DLongGDL>( shadesIx ); doShade=true;
-      } else {
-        // Get COLOR from PLOT system variable
-        DStructGDL* pStruct=SysVar::P();   //MUST NOT BE STATIC, due to .reset 
-        shades=new DLongGDL( 1, BaseGDL::NOZERO );
-        shades_guard.Init ( shades ); // delete upon exit
-        shades=static_cast<DLongGDL*>(pStruct->GetTag(pStruct->Desc()->TagIndex("COLOR"), 0)); doShade=false;
-      }
-      if (doShade) Warning ( "SHADE_SURF: SHADES array ignored, shading with current color table." );
-        return false;
-    } 
-
-  private:
-    void old_body (EnvT* e, GDLGStream* actStream) // {{{
-    {
-      //T3D
-      static int t3dIx = e->KeywordIx( "T3D");
-      bool doT3d=(e->KeywordSet(t3dIx)|| T3Denabled());
-      //ZVALUE
-      static int zvIx = e->KeywordIx( "ZVALUE");
-      DDouble zValue=0.0;
-      e->AssureDoubleScalarKWIfPresent ( zvIx, zValue );
-      zValue=min(zValue,0.999999); //to avoid problems with plplot
-      //SAVE
-      static int savet3dIx = e->KeywordIx( "SAVE");
-      bool saveT3d=e->KeywordSet(savet3dIx);
-      //NODATA
-      static int nodataIx = e->KeywordIx( "NODATA");
-      nodata=e->KeywordSet(nodataIx);
-
-      // [XYZ]STYLE
-      DLong xStyle=0, yStyle=0, zStyle=0; ;
-      gdlGetDesiredAxisStyle(e, XAXIS, xStyle);
-      gdlGetDesiredAxisStyle(e, YAXIS, yStyle);
-      gdlGetDesiredAxisStyle(e, ZAXIS, zStyle);
-
       //check here since after AutoIntvAC values will be good but arrays passed
       //to plplot will be bad...
-      if ( xLog && xStart<=0.0 )
-      {
-        Warning ( "SHADE_SURF: Infinite x plot range." );
-        nodata=true;
-      }
-      if ( yLog && yStart<=0.0 )
-      {
-        Warning ( "SHADE_SURF: Infinite y plot range." );
-        nodata=true;
-      }
-      if ( zLog && zStart<=0.0 ) Warning ( "SHADE_SURF: Infinite z plot range." );
-
-
-      if ( ( xStyle&1 )!=1 )
-      {
-        PLFLT intv=gdlAdjustAxisRange (e, XAXIS, xStart, xEnd, xLog );
-      }
-
-      if ( ( yStyle&1 )!=1 )
-      {
-        PLFLT intv=gdlAdjustAxisRange (e, YAXIS, yStart, yEnd, yLog );
-      }
+      if (xLog && xStart <= 0.0) Warning("SURFACE: Infinite x plot range.");
+      if (yLog && yStart <= 0.0) Warning("SURFACE: Infinite y plot range.");
+      if (zLog && zStart <= 0.0) Warning("SURFACE: Infinite z plot range.");
 
       static int MIN_VALUEIx = e->KeywordIx( "MIN_VALUE");
       static int MAX_VALUEIx = e->KeywordIx( "MAX_VALUE");
 
-      bool hasMinVal=e->KeywordPresent(MIN_VALUEIx);
-      bool hasMaxVal=e->KeywordPresent(MAX_VALUEIx);
-      DDouble minVal=datamin;
-      DDouble maxVal=datamax;
+      hasMinVal = e->KeywordPresent(MIN_VALUEIx);
+      hasMaxVal = e->KeywordPresent(MAX_VALUEIx);
+      minVal = datamin;
+      maxVal = datamax;
       e->AssureDoubleScalarKWIfPresent ( MIN_VALUEIx, minVal );
       e->AssureDoubleScalarKWIfPresent ( MAX_VALUEIx, maxVal );
 
@@ -275,87 +220,153 @@ namespace lib
         zEnd=min(zEnd,maxVal);
       }
 
-      // then only apply expansion  of axes:
-      if ( ( zStyle&1 )!=1 )
-      {
-        PLFLT intv=gdlAdjustAxisRange ( e, ZAXIS, zStart, zEnd, zLog );
-      }
+      //Box adjustement:
+      gdlAdjustAxisRange(e, XAXIS, xStart, xEnd, xLog);
+      gdlAdjustAxisRange(e, YAXIS, yStart, yEnd, yLog);
+      gdlAdjustAxisRange(e, ZAXIS, zStart, zEnd, zLog);
+
+        return false; //do not abort
+    } 
+
+  private:
+    bool prepareDrawArea (EnvT* e, GDLGStream* actStream) // {{{
+    {
+      static int savet3dIx = e->KeywordIx("SAVE");
+      bool saveT3d = e->KeywordSet(savet3dIx);
 
       // background BEFORE next plot since it is the only place plplot may redraw the background...
-      gdlSetGraphicsBackgroundColorFromKw ( e, actStream ); //BACKGROUND
+      gdlSetGraphicsBackgroundColorFromKw ( e, actStream );
+      //start a plot
       gdlNextPlotHandlingNoEraseOption(e, actStream);     //NOERASE
-        // set the PLOT charsize before computing box, see plot command.
+
+      // viewport and world coordinates
+      // set the PLOT charsize before setting viewport (margin depend on charsize)
       gdlSetPlotCharsize(e, actStream);
+      zValue=gdlSetViewPortAndWorldCoordinates(e, actStream, xStart, xEnd, xLog, yStart, yEnd, yLog, zStart, zEnd, zLog, zValue);
 
       // Deal with T3D options -- either present and we have to deduce az and alt contained in it,
       // or absent and we have to compute !P.T from az and alt.
 
-      PLFLT alt=30.0;
-      PLFLT az=30.0;
+      PLFLT scale[3]={SCALEBYDEFAULT,SCALEBYDEFAULT,SCALEBYDEFAULT};
+      if (!doT3d) { // or absent and we have to compute !P.T from az and alt.
       //set az and ax (alt)
+        DFloat az_change = az;
+        static int AZIx = e->KeywordIx("AZ");
+        e->AssureFloatScalarKWIfPresent(AZIx, az_change);
+        az = az_change;
+        
       DFloat alt_change=alt;
       static int AXIx=e->KeywordIx("AX");
       e->AssureFloatScalarKWIfPresent(AXIx, alt_change);
       alt=alt_change;
-
-      alt=fmod(alt,360.0); //restrict between 0 and 90 for plplot!
-      if (alt > 90.0 || alt < 0.0)
-      {
-        e->Throw ( "SHADE_SURF: AX restricted to [0-90] range by plplot (fix plplot!)" );
+        alt=atan2(sin(alt * GDL_PI/180.0), cos(alt * GDL_PI/180.0)) * 180.0/GDL_PI;
+        alt=fmod((alt+360),360.0);
+        if (alt > 90 && alt <= 270) {
+          az+=180.;
+          if (alt > 180) {below=true; alt-=180; alt*=-1;} else alt=180-alt;
+        } else if (alt > 270) {
+          below=true;
+          alt=-(360.-alt);
       }
-      DFloat az_change=az;
-      static int AZIx=e->KeywordIx("AZ");
-      e->AssureFloatScalarKWIfPresent(AZIx, az_change);
-      az=az_change;
+        //Compute special transformation matrix for the BOX and give it to the driver
+        gdlBox3d=gdlDefinePlplotRotationMatrix( az, alt, scale, saveT3d);
+        Guard<BaseGDL> g(gdlBox3d);
+        GDL_3DTRANSFORMDEVICE T3DForAXes;
+        for (int i = 0; i < 16; ++i)T3DForAXes.T[i] =(*gdlBox3d)[i];
+        T3DForAXes.zValue = (std::isfinite(zValue))?zValue:0;
+        gdlStartSpecial3DDriverTransform(actStream,T3DForAXes);
+        Current3DMatrix=static_cast<DDouble*>(gdlBox3d->DataAddr());
+      } else {
+        //just ask for P.T3D transform with the driver:
+        bool ok=gdlInterpretT3DMatrixAsPlplotRotationMatrix(az, alt, ay, scale, axisExchangeCode, below);
+        if (!ok) Warning ( "SHADE_SURF: Illegal 3D transformation." );
+        gdlStartT3DMatrixDriverTransform(actStream, zValue);
 
-      //now we are in plplot different kind of 3d
-      DDoubleGDL* plplot3d;
-      DDouble ay, scale; //not useful at this time
-      if (doT3d) //convert to this world...
-      {
-
-        plplot3d=gdlConvertT3DMatrixToPlplotRotationMatrix(zValue, az, alt, ay, scale, axisExchangeCode);
-        if (plplot3d == NULL)
-        {
-          e->Throw ( "SHADE_SURF: Illegal 3D transformation." );
-        }
+        DStructGDL* pStruct = SysVar::P(); //MUST NOT BE STATIC, due to .reset
+        static unsigned tTag = pStruct->Desc()->TagIndex("T");
+        Current3DMatrix = static_cast<DDouble*> (pStruct->GetTag(tTag, 0)->DataAddr());
+        
       }
-      else //make the transformation ourselves
-      {
-        scale=1/sqrt(3.0);
-        //Compute transformation matrix with plplot conventions:
-        plplot3d=gdlComputePlplotRotationMatrix( az, alt, zValue,scale);
-        // save !P.T if asked to...
-        if (saveT3d) //will use ax and az values...
-        {
-          DDoubleGDL* t3dMatrix=plplot3d->Dup();
-          SelfTranspose3d(t3dMatrix);
-          DStructGDL* pStruct=SysVar::P();   //MUST NOT BE STATIC, due to .reset 
-          static unsigned tTag=pStruct->Desc()->TagIndex("T");
-          for (int i=0; i<t3dMatrix->N_Elements(); ++i )(*static_cast<DDoubleGDL*>(pStruct->GetTag(tTag, 0)))[i]=(*t3dMatrix)[i];
-          GDLDelete(t3dMatrix);
-        }
+      // We could have kept the old code where the box was written by plplot's box3(), but it would not be compatible with the rest of the eventual other (over)plots
+      //Draw axes with normal color!
+      //Should draw 3d mesh before axes
+      gdlSetGraphicsForegroundColorFromKw ( e, actStream ); //COLOR
+      //write OUR box using our 3D PLESC tricks:
+      gdlBox3(e, actStream, xStart, xEnd, xLog, yStart, yEnd, yLog, zStart, zEnd, zLog, zValue);
+       //reset driver to 2D plotting routines, further 3D is just plplot drawing a mesh.
+      gdlStop3DDriverTransform(actStream); 
+      
+      //we now pass EVERYTHING in normalized coordinates w/o clipping and set up a transformation to have plplot mesh correct on the 2D vpor.
+      //however we need to check that clip values are OK to reproduce IDL's behaviour (no plot at all):
+      //SHADE_SURF IGNORES the CLIP/NOCLIP stuff --- check by yourself.
+//      if (gdlTestClipValidity(e, actStream)) return true; //note clip meaning is normal
+      const COORDSYS coordinateSystem = DATA;
+      SelfConvertToNormXYZ(xStart, xLog, yStart, yLog, zStart, zLog, coordinateSystem); 
+      SelfConvertToNormXYZ(xEnd, xLog, yEnd, yLog, zEnd, zLog, coordinateSystem);
+      if (hasMinVal) ConvertToNormZ(1, &minVal, zLog, coordinateSystem);
+      if (hasMaxVal) ConvertToNormZ(1, &maxVal, zLog, coordinateSystem);
+      //WARNING Following sets coordinateSystem=NORM!
+      COORDSYS coordinateSystem2 = DATA;
+      SelfConvertToNormXY(xVal, xLog, yVal, yLog, coordinateSystem2); //always DATA for PLOT X,Y values
+      ConvertToNormZ(xEl*yEl, (DDouble*) zVal->DataAddr(), zLog, coordinateSystem);
+// the mapping between data 3-d and world 3-d coordinates is given by:
+//
+//   x = xmin   =>   wx = -0.5*basex
+//   x = xmax   =>   wx =  0.5*basex
+//   y = ymin   =>   wy = -0.5*basey
+//   y = ymax   =>   wy =  0.5*basey
+//   z = zmin   =>   wz =  0.0
+//   z = zmax   =>   wz =  height
+      
+//Due to a bug in plplot, shading (here) and surface (in plotting_surface) do not behave similarly WRT the vpor and wind.
+//This is the good version for shade_surf, shifting the shaded surface by som amount in the 3DDriverTransform of the driver, not the plplot library.     
+      actStream->vpor(0,1,0,1);
+      actStream->wind(-0.5/scale[0],0.5/scale[0],-0.5/scale[1],0.5/scale[1]); //mandatory: to center in (0,0,0) for 3D Matrix rotation.
+      if (below) { actStream->w3d(1,1,1,0,1,0,1,0,1, -alt, az);
+        DDouble xp = 0;
+        DDouble yp1 = 0;
+        DDouble yp2 = 0;
+        Matrix3DTransformXYZval(0, 0, 0, &xp, &yp1,Current3DMatrix);
+        Matrix3DTransformXYZval(0, 0, 0.5, &xp, &yp2,Current3DMatrix);
+        gdlShiftYaxisUsing3DDriverTransform(actStream, 1-(yp1 - yp2), true);
+      } else {
+        actStream->w3d(1,1,1,0,1,0,1,0,1, alt, az); //mandatory: in order to have shades plotted correctly, z must go from 0 to 1, not -0.5 to 0.5
+        //as the code in plplot prevents negative "normalized" values.
+        // To insure this (and shade_surf) to work in all cases, we must rely on the 3DDriverTransform, once again, to shift the [0,1] plot in [-0.5, 0.5]
+        // 
+        //compute vertical displacement of point [0,0,0] in projected coordinates between zv=0 and zv=0.5
+        DDouble xp = 0;
+        DDouble yp1 = 0;
+        DDouble yp2 = 0;
+        Matrix3DTransformXYZval(0, 0, 0, &xp, &yp1,Current3DMatrix);
+        Matrix3DTransformXYZval(0, 0, 0.5, &xp, &yp2,Current3DMatrix);
+        gdlShiftYaxisUsing3DDriverTransform(actStream, yp1 - yp2, false);
+      }
+      
+      return false;
       }
 
-      if ( gdlSet3DViewPortAndWorldCoordinates(e, actStream, plplot3d, xLog, yLog,
-        xStart, xEnd, yStart, yEnd, zStart, zEnd, zLog)==FALSE ) return;
+void applyGraphics(EnvT* e, GDLGStream * actStream) {
+      //NODATA
+      static int nodataIx = e->KeywordIx("NODATA");
+      nodata = e->KeywordSet(nodataIx);
 
-      if (xLog) xStart=log10(xStart);
-      if (yLog) yStart=log10(yStart);
-      if (zLog) zStart=log10(zStart);
-      if (xLog) xEnd=log10(xEnd);
-      if (yLog) yEnd=log10(yEnd);
-      if (zLog) zEnd=log10(zEnd);
-
-       actStream->w3d(scale,scale,scale*(1.0-zValue),
-                     xStart, xEnd, yStart, yEnd, zStart, zEnd,
-                     alt, az);
-
-
-      //Draw 3d mesh before axes
       // PLOT ONLY IF NODATA=0
-      if (!nodata)
-      {
+      if (!nodata) {
+        //SHADES : not supported yet since plplot does not honor it correctly --- see documentation.
+        // shades //      static int shadesIx = e->KeywordIx("SHADES");
+        // shades //      bool doShade=false;
+        // shades //      DLongGDL* shadevalues=NULL;
+        // shades //      if (e->GetKW(shadesIx) != NULL) {
+        // shades //        shadevalues = e->GetKWAs<DLongGDL>(shadesIx);
+        // shades //        doShade=true;
+        // shades //      }
+        // Get decomposed value for shades
+        // shades //      if (doShade && decomposed==0) actStream->SetColorMap1Table(shadevalues->N_Elements(), shadevalues, decomposed); 
+        // shades //      else if (doShade && decomposed==1) actStream->SetColorMap1DefaultColors(256,  decomposed );
+        // shades //      else 
+        
+        DLong decomposed = actStream->ForceColorMap1Ramp(0.33);
         //use of intermediate map for correct handling of blanking values and nans.
         PLFLT ** map;
         actStream->Alloc2dGrid( &map, xEl, yEl);
@@ -364,22 +375,10 @@ namespace lib
           for ( SizeT j=0; j<yEl; j++)
           { //plplot does not like NaNs and any other terribly large gradient!
             PLFLT v=(*zVal)[k++];
-            if (zLog)
-            {
-              v= log10(v);
-              PLFLT miv=log10(minVal);
-              PLFLT mav=log10(maxVal);
-              if ( !isfinite(v) ) v=miv;
-              if ( hasMinVal && v < miv) v=miv;
-              if ( hasMaxVal && v > mav) v=mav;
-            }
-            else
-            {
               if ( !isfinite(v) ) v=minVal;
               if ( hasMinVal && v < minVal) v=minVal;
               if ( hasMaxVal && v > maxVal) v=maxVal;
-            }
-            map[i][j] = v;
+            map[i][j] = (below)?1-v:v;
           }
         }
         // 1 types of grid only: 1D X and Y.
@@ -392,62 +391,33 @@ namespace lib
         cgrid1.yg = yg1;
         cgrid1.nx = xEl;
         cgrid1.ny = yEl;
-        for ( SizeT i=0; i<cgrid1.nx; i++ ) cgrid1.xg[i] = (*xVal)[i];
-        for ( SizeT i=0; i<cgrid1.ny; i++ ) cgrid1.yg[i] = (*yVal)[i];
-        //apply projection transformations:
-        //not until plplot accepts 2D X Y!
-        //apply plot options transformations:
-        if (xLog) for ( SizeT i=0; i<cgrid1.nx; i++ ) cgrid1.xg[i] = cgrid1.xg[i]>0?log10(cgrid1.xg[i]):1E-12;  // #define EXTENDED_DEFAULT_LOGRANGE 12
-        if (yLog) for ( SizeT i=0; i<cgrid1.ny; i++ ) cgrid1.yg[i] = cgrid1.yg[i]>0?log10(cgrid1.yg[i]):1E-12;
-
-        // Important: make all clipping computations BEFORE setting graphic properties (color, size)
-        static int NOCLIPIx = e->KeywordIx("NOCLIP");
-        static int CLIPIx = e->KeywordIx("CLIP");
-        bool doClip=(e->KeywordSet(CLIPIx)||e->KeywordSet(NOCLIPIx));
-        bool stopClip=false;
-        if ( doClip )  if ( startClipping(e, actStream)==false ) stopClip=true;
-
-        gdlSetGraphicsForegroundColorFromKw ( e, actStream );
+        for ( SizeT i=0; i<xEl; i++ ) cgrid1.xg[i] = (*xVal)[i];
+        for ( SizeT i=0; i<yEl; i++ ) cgrid1.yg[i] = (*yVal)[i];
+        
         //mesh option
-        PLINT meshOpt;
-        meshOpt=(doShade)?MAG_COLOR:0;
-        static int SKIRTIx = e->KeywordIx("SKIRT");
-        if (e->KeywordSet ( SKIRTIx )) meshOpt+=DRAW_SIDES;
+        PLINT meshOpt=0;
+// shades //        meshOpt=(doShade)?MAG_COLOR:0;
 
-        // Get decomposed value for shades
-        DLong decomposed=GraphicsDevice::GetDevice()->GetDecomposed();
-        if (doShade) actStream->SetColorMap1DefaultColors(256,  decomposed );
-        else actStream->SetColorMap1Ramp(decomposed, 0.5); 
         //position of light Source. Plplot does not use only the direction of the beam but the position of the illuminating
         //source. And its illumination looks strange. We try to make the ill. source a bit far in the good direction.
         PLFLT sun[3];
-        sun[0]=xStart+(xEnd-xStart)*(0.5+lightSourcePos[0]);
-        sun[1]=yStart+(yEnd-yStart)*(0.5+lightSourcePos[1]);
-        sun[2]=zStart+(zEnd-zStart)*((1.0-zValue)+lightSourcePos[2]);
+        sun[0]=lightSourcePos[0]*1E10;
+        sun[1]=lightSourcePos[1]*1E10;
+        sun[2]=lightSourcePos[2]*1E10;
         actStream->lightsource(sun[0],sun[1],sun[2]);
         actStream->surf3d(xg1,yg1,map,cgrid1.nx,cgrid1.ny,meshOpt,NULL,0);
 
-        if (stopClip) stopClipping(actStream);
-//Clean alllocated data struct
+//Clean allocated data struct
         delete[] xg1;
         delete[] yg1;
         actStream->Free2dGrid(map, xEl, yEl);
+        if (decomposed > 0) GraphicsDevice::GetDevice()->Decomposed(true);
       }
-      //Draw axes with normal color!
-      gdlSetGraphicsForegroundColorFromKw ( e, actStream ); //COLOR
-      gdlBox3(e, actStream, xStart, xEnd, yStart, yEnd, zStart, zEnd, xLog, yLog, zLog, true);
     } 
 
-  private:
-
-    void call_plplot (EnvT* e, GDLGStream* actStream) 
-    {
-    } 
-
-  private:
-
     virtual void post_call (EnvT*, GDLGStream* actStream) 
     {
+      gdlStop3DDriverTransform(actStream); 
       actStream->lsty(1);//reset linestyle
       actStream->sizeChar(1.0);
     } 
@@ -462,6 +432,9 @@ namespace lib
 
  void set_shading(EnvT* e)
  {
+   lightSourcePos[0]=0;
+   lightSourcePos[1]=0;
+   lightSourcePos[2]=1;
     DDoubleGDL *light;
     static int lightIx=e->KeywordIx ( "LIGHT" );
     if ( e->GetKW ( lightIx )!=NULL )
