@@ -102,6 +102,15 @@
 #define TRACEOMP( file, line) 
 #endif
 
+//#define TRACE_OPCALLS
+#undef TRACE_OPCALLS
+
+#if defined(TRACE_OPCALLS)
+#define TRACE_ROUTINE(func,file,line) std::cout << func << "\t" << file << "\t" << line << std::endl;
+#else
+#define TRACE_ROUTINE(func,file,line)
+#endif
+
 // SA: fixing bug no. 3296360
 typedef unsigned long long int      SizeT;
 typedef long long int RangeT;
